import {
  Loader, Metadata, IDictionary, dayjs as moment, I18n, Http,
} from '@zeedhi/core';
import { Form, DialogService, LoadingService } from '@zeedhi/common';

export class ImportacaoController {
  private utilities = Loader.getInstance('Utilities');

  public unidades: IDictionary = this.utilities.getItemLocalStorage('Units') || [];

  public agrupamentosProdutos: IDictionary = this.utilities.getItemLocalStorage('GruposProduto') || [];

  public importar() {
    const form = Metadata.getInstance<Form>('formImportacao');
    if (form.validate()) {
      const { value } = form;
      if (this.utilities.validateInterval(value.INTERVAL[0], value.INTERVAL[1])) {
        if (this.hasImportedData() === false) {
          this.continueImportacao();
        }
      }
    }
  }

  public continueImportacao() {
    if (this.validateActiveUser()) {
      LoadingService.show(' ');
      this.clearImportData().then(() => {
        this.importRequestProductsData();
        // this.importProductsData();
        // this.importGeneralData();
        // this.finalizeImportProcess();
        LoadingService.hide();
      });
    }
  }

  public importRequestProductsData() {
    const form = Metadata.getInstance<Form>('formImportacao');
    const row = {
      dataInicial: moment(form.value.INTERVAL[0]).format('DD/MM/YYYY'),
      dataFinal: moment(form.value.INTERVAL[1]).format('DD/MM/YYYY'),
      periodo: 5,
      filiais: form.value.UNIDADE,
    };
    return Http.post('/ProdutosSolicitacao', { requestType: 'Row', row }).then((allData) => {
      const result = allData.flat;
      this.utilities.setItemLocalStorage('requestedProducts', result);
      const reducedProds: IDictionary = [];
      result.forEach((product: IDictionary) => {
        reducedProds.push({
          CDFILSOLI: product.CDFILSOLI,
          CDPRODUTO: product.CDPRODUTO,
          NMPRODUTO: product.NMPRODUTO,
        });
      });
      this.removeDuplicateProducts(reducedProds);
      this.utilities.setItemLocalStorage('reducedUniqueRequestedProducts', reducedProds);
    });
  }

  public removeDuplicateProducts(products: IDictionary) {
    this.utilities.spliceOnCondition(products, (product: IDictionary) => {
      let ocurrences = 0;
      products.forEach((reduceProd: IDictionary) => {
        const reduceProdData = [reduceProd.CDFILSOLI, reduceProd.CDPRODUTO];
        const currentProductData = [product.CDFILSOLI, product.CDPRODUTO];

        if (JSON.stringify(reduceProdData) === JSON.stringify(currentProductData)) {
          ocurrences += 1;
        }
      });
      return ocurrences > 1;
    });
  }

  public async clearImportData() {
    const importedDataToClean = [
      'AF', ' CardapioEfetivos', 'ClientesDoServicoEfetivos',
      'DatasCardapioEfetivos', 'DatasEntregaSolicitacao', 'DatasInventario',
      'Justificativas', 'Parametros', 'ProximoCodigoInventario', 'ProximoCodigoProdutoInventario',
      'ServicosCardapioEfetivos', 'Solicitacao', 'UltimoInventario',
      'availableUnitsToChange', 'reducedUniqueRequestedProducts', 'requestedProducts',
    ];

    await importedDataToClean.forEach((store) => {
      localStorage.setItem(store, '');
    });
  }

  public async validateActiveUser() {
    const filter = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: localStorage.getItem('CDOPERADOR'),
      },
    ];
    await Http.post('/OperadorAtivo', { requestType: 'FilterData', filter }).then((result: IDictionary) => {
      if (!result) {
        this.utilities.showAlert('inactiveUser', 'INACTIVE_USER', 'error');
        // logout
        return false;
      }
      return true;
    });
  }

  public hasImportedData() {
    if (localStorage.getItem('hasImportedData')) {
      // this.utilities.showAlert('alreadyImported', 'HAS_IMPORTED_DATA', 'e')
      DialogService.show({
        name: 'confirmImport',
        type: 'warning',
        title: I18n.translate('ATTENTION'),
        text: I18n.translate('HAS_IMPORTED_DATA'),
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: I18n.translate('YES'),
            events: { click: () => this.continueImportacao() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: I18n.translate('NO'),
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
    return false;
  }
}
