var DashboardController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Translation method.
     */
    const i18n = ScreenService.i18n;

    /**
     * Instructions to be executed when enter at 
     * dashboardWidget.
     * 
     * @param {Widget} dashboardWidget
     */
    this.onEnterMainDashboard = dashboardWidget => {
        setDefaultValuesOnCurrentRow(dashboardWidget);
        buildUnitAndLogCards();
    };

    /**
     * Set values on dashboard currentRow, used to
     * format the welcome message
     * 
     * @param {Widget} dashboardWidget
     */
    const setDefaultValuesOnCurrentRow = dashboardWidget => {
        dashboardWidget.currentRow = {
            lastImport: getLocalVar('hasImportedData') ? moment(getLocalVar('lastImport')).format('DD/MM/YYYY HH:mm') : '-'
        };
    };

    /**
     * Open unit change if have imported data.
     */
    this.openChangeUnit = () => {
        if (!getLocalVar('hasImportedData')) {
            ScreenService.showMessage(i18n('HAVE_TO_IMPORT_DATA')).then(() => {
                UIManager.openContainer('dataImport');
            });
        } else {
            UIManager.openContainer('changeUnit');
        }
    };

    /**
     * Build Unit and error log cards information
     * to show in main dashboard.
     * 
     * @returns {Promise}
     */
    const buildUnitAndLogCards = () => {
        const dsbWidget = templateManager.container.getWidget('mainDashboard');
        const unitCard = dsbWidget.horizontalDashboards.dashboard.dataSource.find(card => card.name == "unitCard");
        const errorLogCard = dsbWidget.horizontalDashboards.dashboard.dataSource.find(card => card.name == "errorLogCard");

        return getCurrentUnitParams().then(params => {
            unitCard.main = unitCard.main.replace('__INFO__', buildUnitCardContent(params));

            return buildLogCardContent(params).then(logCard => {
                errorLogCard.main = errorLogCard.main.replace('__INFO__', logCard);
                templateManager.updateTemplate();
            });
        });
    };

    /**
     * Build info to show at unit card in main
     * screen according to logged unit parameters.
     * 
     * @param {Object} params 
     * @returns {String}
     */
    const buildUnitCardContent = params => {
        let content = '';
        let userData = getLocalVar('userData');

        if (getLocalVar('hasImportedData')) {

            if (userData.CDFILIAL === userData.CDFILCENTPME) {
                content += `${i18n("UNIT_IS_MEALS_CENTER")}<br>`;
            } else if (!userData.CDFILCENTPME) {
                content += `${i18n("UNIT_HAVE_NO_MEALS_CENTER")}<br>`;
            }

            if (params) {
                if (params.IDUTILALMOX === 'S') {
                    content += `${i18n("UNIT_HAVE_STOCK_CTRL_WAREHOUSE")}<br>`;
                }

                if (params.IDUTILLOTE === 'S') {
                    content += `${i18n("UNIT_HAVE_BATCH_STOCK_CTRL")}<br>`;
                }
            }

        }

        if (!content.length) {
            content = i18n("NO_INFO_TO_SHOW");
        }

        return '<br>' + content;
    };

    /**
     * Build info to show at error log card in main
     * screen according to logged unit parameters.
     * 
     * @param {Object} params 
     * 
     * @returns {Promise<String>}
     */
    const buildLogCardContent = params => {
        return SchoolMealsUtil.getIDBVar('LogSolicitacao').then(log => {
            let content = '';

            if (log && log.length) {
                content += `<br>${i18n('LAST_ERROR')}<br>${log[0].DATAHORA}`;
            }

            if (!content.length) {
                content = i18n("NO_INFO_TO_SHOW");
            }

            return content;
        });
    };

    /**
     * Get logged unit related parameters
     * from 'Parametros' datasource.
     * 
     * @returns {Promise}
     */
    const getCurrentUnitParams = () => SchoolMealsUtil.getIDBVar('Parametros')
        .then((parameters) => parameters && parameters.find(param => param.CDFILIAL == getLocalVar('userData').CDFILIAL));
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DashboardController', DashboardController);
});
