var DataImportController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, DataImportService, LoginService) {

    /**
     * Make all validations to check if is valid to 
     * import data from selected units in the form.
     */
    this.initDataImportProcess = () => {
        let dataImportForm = templateManager.container.getWidget('dataImport');
        let validationObj = Object.assign({}, dataImportForm.currentRow);

        validateUserRelatedUnits(validationObj).then(relatedUnits => {
            validationObj.relatedUnits = relatedUnits;
            validationObj.mappedRelatedUnits = relatedUnits.map(unit => unit.CDFILIAL);

            validateIntervalToImportData(dataImportForm)
                .then(hasImportedData)
                .then(validateActiveUser)
                .then(() => {
                    importData(validationObj);
                }).catch(error => {
                    console.warn(error);
                });
        });
    };

    /**
     * Validate if all selected units are related
     * to current user.
     * 
     * @param {Object} validationObj Data import parameters.
     * 
     * @return {Promise}
     */
    const validateUserRelatedUnits = validationObj => {
        let selectedUnits = validationObj.CDFILIAL;

        if (selectedUnits && selectedUnits.length) {
            return DataImportService.getUserRelatedUnits(getLocalVar('userData').CDOPERADOR, selectedUnits).then(units => {
                if (units.length !== selectedUnits.length) {
                    return ScreenService.confirmMessage(ScreenService.i18n('NOT_RELATED_UNITS')).then(() => units);
                } else {
                    return Promise.resolve(units);
                }
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n('SELECT_UNIT_TO_IMPORT'));
            return Promise.reject();
        }
    };

    /**
     * Validate selected interval to import data.
     * Selected interval is required, and 
     * must be less or equal 31 days.
     * 
     * @param {Widget} dataImportForm 
     */
    const validateIntervalToImportData = dataImportForm => {
        let start = dataImportForm.currentRow.INTERVAL_START;
        let end = dataImportForm.currentRow.INTERVAL_END;

        if (!start || !end) {
            ScreenService.showMessage(ScreenService.i18n('SELECT_COMPLETE_INTERVAL'));
            return Promise.reject();
        } else {
            start = moment(start);
            end = moment(end).add(1, "days");
            let diff = end.diff(start, "days");

            if (diff > 31) {
                ScreenService.showMessage(ScreenService.i18n('SELECT_31_DAYS_INTERVAL'));
                return Promise.reject();
            } else {
                return Promise.resolve();
            }
        }
    };

    /**
     * Check if already has imported data.
     * 
     * @return {Promise} resolved promise
     * if should continue importation process. 
     */
    const hasImportedData = () => {
        if (getLocalVar('hasImportedData')) {
            return ScreenService.confirmMessage(ScreenService.i18n('HAS_IMPORTED_DATA'));
        } else {
            return Promise.resolve();
        }
    };

    /**
     * Check if current user is active on system.
     * If it's not, so should logout user.
     * 
     * @return {Promise}
     */
    const validateActiveUser = () => {
        let userMail = getLocalVar('userData').DSEMAILOPER;
        let nrorg = getLocalVar('userData').NRORG;

        return LoginService.checkActiveUser(userMail, nrorg).then(isActive => {
            if (isActive) {
                return Promise.resolve();
            } else {
                ScreenService.showMessage(ScreenService.i18n('INACTIVE_USER'));
                LoginService.logout();
                return Promise.reject();
            }
        });
    };

    /**
     * Import application data according to parameters
     * selected in data import screen.
     * 
     * @param {Object} params Parameters to execute entire
     * process.
     */
    const importData = (params) => {
        return DataImportService.clearImportData() //@todo finalize
            .then(() => importRequestProductsData(params))
            .then(importProductsData)
            .then(importGeneralData)
            .then(finalizeImportProcess);
    };

    /**
     * Import Products of system requests from backend.
     * 
     * @param {Object} params 
     * 
     * @return {Promise}
     */
    const importRequestProductsData = params => {
        let requestedProductsParams = {
            nrorg: getLocalVar('userData').NRORG,
            start: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
            end: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
            period: 5
        };

        return DataImportService.importRequestProductsData(requestedProductsParams, params.relatedUnits).then(requestedProducts => {
            SchoolMealsUtil.setIDBVar('requestedProducts', requestedProducts, true);

            let reducedProds = requestedProducts.map(product => Object.assign({
                CDFILSOLI: product.CDFILSOLI,
                CDPRODUTO: product.CDPRODUTO,
                NMPRODUTO: product.NMPRODUTO
            }));

            removeDuplicateProducts(reducedProds);
            SchoolMealsUtil.setIDBVar('reducedUniqueRequestedProducts', reducedProds, true);

            return params;
        });
    };

    /**
     * Remove all duplicates of products in a array, using
     * SchoolMealsUtil -> spliceOnCondition function.
     * 
     * @param {Array} productArray 
     */
    const removeDuplicateProducts = productArray => {
        spliceOnCondition(productArray, product => {
            let ocurrences = 0;

            productArray.forEach(reduceProd => {
                let reduceProdData = [reduceProd.CDFILSOLI, reduceProd.CDPRODUTO];
                let currentProductData = [product.CDFILSOLI, product.CDPRODUTO];

                if (JSON.stringify(reduceProdData) == JSON.stringify(currentProductData)) {
                    ocurrences++;
                }
            });

            return ocurrences > 1;
        });
    };

    /**
     * Import system products data.
     * 
     * @param {Object} params Search data params.
     * 
     * @returns {Promise}
     */
    const importProductsData = params => {
        return DataImportService.importProductsData(params, params.relatedUnits).then(products => {
            let productsObject = {};

            products.forEach(product => {
                if (productsObject.hasOwnProperty(product.CDPRODUTO)) {
                    productsObject[product.CDPRODUTO].units.push(product.CDFILIAL);
                } else {
                    productsObject[product.CDPRODUTO] = product;
                    productsObject[product.CDPRODUTO].units = [product.CDFILIAL];
                }
            });

            SchoolMealsUtil.setIDBVar('productsDataset', products, true);
            SchoolMealsUtil.setIDBVar('productsDatasetObject', productsObject, true);

            params.productsData = productsObject;

            return params;
        });
    };

    /**
     * Import general data, related with working of 
     * all parts of system.
     * 
     * @param {Object} params 
     */
    const importGeneralData = params => {
        params.period = 5;

        return DataImportService.importGeneralData(params).then(generalData => {
            Object.keys(generalData).forEach(datasetName => {
                SchoolMealsUtil.setIDBVar(datasetName, generalData[datasetName], true);
            });

            return params;
        });

    };

    /**
     * Finalize the importation process of all system
     * necessary data.
     * 
     * @param {Object} params Data importation parameters.
     */
    const finalizeImportProcess = params => {
        setLocalVar('hasImportedData', true);
        setLocalVar('lastImport', moment());

        let userData = getLocalVar('userData');
        let currentUnit = userData.CDFILIAL;
        let msg = ScreenService.i18n('IMPORTED_DATA');

        userData = Object.assign(userData, {
            INTERVAL_START: params.INTERVAL_START,
            INTERVAL_END: params.INTERVAL_END
        });
        setLocalVar('userData', userData);

        if (!Object.keys(params.productsData).length) {
            msg += "<br>" + ScreenService.i18n('NO_IMPORTED_PRODUCT');
        }

        setUnitChangeDataSource(params);
        setSystemEmptyDataSources();

        if (params.mappedRelatedUnits.includes(currentUnit)) {
            SchoolMealsUtil.changeUnit(currentUnit);

            ScreenService.openWindow('dashboard').then(function() {
                ScreenService.showMessage(msg);
            });
        } else {
            setLocalVar('CDFILIAL', false);
            ScreenService.showMessage(msg).then(() => {
                ScreenService.openWindow('changeUnit').then(() => {
                    ScreenService.showMessage(ScreenService.i18n('PLEASE_CHANGE_UNIT'));
                });
            });
        }
    };

    /**
     * Set units dataSource, used to change
     * unit.
     * 
     * @param {Object} params 
     */
    const setUnitChangeDataSource = params => {
        let availableUnitsToChange = [];

        SchoolMealsUtil.getIDBVar('Units', true).then(units => {
            params.CDFILIAL.forEach(unitCode => {
                availableUnitsToChange.push(units.find(unit => unit.CDFILIAL == unitCode));
            });

            SchoolMealsUtil.setIDBVar('availableUnitsToChange', availableUnitsToChange, true);
        });
    };

    /**
     * Set all unseteds system empty dataSources.
     */
    const setSystemEmptyDataSources = () => {
        SchoolMealsUtil.setIDBVar('ProdutosSolicitacao', []);
        SchoolMealsUtil.setIDBVar('ProdutosInventario', []);
        SchoolMealsUtil.setIDBVar('ProdutosExcluidos', []);
        SchoolMealsUtil.setIDBVar('NovaSolicitacao', []);
        SchoolMealsUtil.setIDBVar('Movopsol', []);
        SchoolMealsUtil.setIDBVar('Inventarios', []);
        SchoolMealsUtil.setIDBVar('nextInventoryProductCode', {});
    };

    /**
     * On change date start, set end to
     * 30 days later.
     * 
     * @param {Object} field
     */
    this.onChangeDateIntervalStart = field => {
        const importForm = field.widget;
        let startDate = moment(field.value());

        startDate.add(30, 'days');

        importForm.currentRow.INTERVAL_END = startDate.format('YYYY-MM-DD');
        templateManager.updateTemplate();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataImportController', DataImportController);
});
