var ManageInventoryController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, ManageInventoryService) {

    /**
     * @var service Reference to ManageInventoryService.
     */
    const service = ManageInventoryService;

    /**
     * Reference to Container's getWidget function. Return 
     * widget object by name.
     * 
     * @param {String} widgetName Name of widget to return.
     *
     * @returns {Object<Widget>} widget
     */
    const getWidget = widgetName => templateManager.container.getWidget(widgetName);
    /**
     * Check if is valid to init screen.
     * Test if have imported data, a logged 
     * unit, and check if is valid to create
     * new inventory registers.
     */
    this.initScreenIfIsValid = () => {

        if (!SchoolMealsUtil.hasImportedData()) {
            openScreenWithDashboardPushState('dataImport').then(service.showMessage('HAVE_TO_IMPORT_DATA'));
        } else if (!SchoolMealsUtil.hasLoggedUnit()) {
            openScreenWithDashboardPushState('changeUnit').then(service.showMessage('PLEASE_CHANGE_UNIT'));
        } else {
            checkIfIsValidCreateNewInventory().then(() => openScreenWithDashboardPushState('manageInventory'));
        }

    };

    /**
     * Check if is valid to create new inventory.
     * Is valid if: Current unit is not a meals center, 
     * if unit have a meals center, if have no stock control
     * by warehouse and have no batch stock control.
     * 
     * If is invalid, show a warning message to user, explaining
     * the error.
     * 
     * @return {Promise}
     */
    const checkIfIsValidCreateNewInventory = () => SchoolMealsUtil.getIDBVar('Parametros').then(parameters => {
        let valid = false;
        let msg = '';

        let filteredParams = parameters.find(param => param.CDFILIAL == getLocalVar('userData').CDFILIAL);

        if (service.isCurrentUnitMealsCenter()) {
            msg = ScreenService.i18n('UNIT_IS_MEALS_CENTER') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else if (!service.currentUnitHaveMealsCenter()) {
            msg = ScreenService.i18n('UNIT_HAVE_NO_MEALS_CENTER') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else if (filteredParams.IDUTILALMOX == 'S') {
            msg = ScreenService.i18n('UNIT_HAVE_STOCK_CTRL_WAREHOUSE') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else if (filteredParams.IDUTILLOTE == 'S') {
            msg = ScreenService.i18n('UNIT_HAVE_BATCH_STOCK_CTRL') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else {
            valid = true;
        }

        if (!valid)
            service.showMessage(msg);

        return valid;
    });

    /**
     * Open screen using ScreenService.openWindow and
     * push state using UIManager.pushState to correctly
     * go back to dashboard when back action is pressed.
     */
    const openScreenWithDashboardPushState = screenName => {
        return ScreenService.openWindow(screenName).then(() => {
            UIManager.pushState(() => {
                ScreenService.openWindow('dashboard');
            });
        });
    };

    /**
     * Open formulary to add new inventory
     * register. Check if is valid to create new
     * inventory before open form.
     */
    this.openFormToAddInventory = () => {
        checkIfIsValidCreateNewInventory().then(valid => {
            if (valid) {
                const newInventoryForm = templateManager.container.getWidget('newInventoryForm');
                newInventoryForm.currentRow = {};
                UIManager.goToWidget(newInventoryForm);
            }
        });
    };

    /**
     * Save new inventory register created by user
     * if all data is filled correctly and there's no
     * inventory with same date.
     */
    this.saveNewInventory = () => {
        const newInventoryForm = templateManager.container.getWidget('newInventoryForm');

        if (validateInventoryToSave(newInventoryForm.currentRow)) {
            checkExistentInventory(newInventoryForm.currentRow).then(existsInventory => {
                if (!existsInventory) {
                    service.insertInventoryRegister(newInventoryForm.currentRow);
                }
            });
        }
    };

    /**
     * Check if all informations are filled,
     * if inventory date is less than or equal to
     * importation date.
     * 
     * Show to user a warning message if something
     * goes wrong.
     * 
     * @param {Object} inventoryRow Object row
     * containing inventory information.
     * 
     * @return {Boolean}
     */
    const validateInventoryToSave = inventoryRow => {
        let isValid = true;

        if (!service.isInventoryRequiredFieldsFilled(inventoryRow)) {
            service.showMessage('FILL_REQUIRED_FIELDS');
            isValid = false;
        } else if (!service.isInventoryDateValid(inventoryRow)) {
            isValid = false;
            service.showMessage('INVALID_INVENTORY_DATE');
        }

        return isValid;
    };

    /**
     * Check existance of a inventory date in local
     * and remote storage.
     * 
     * Show to user a warning message if something
     * goes wrong.
     * 
     * @param {Object} inventoryRow Object row
     * containing inventory information.
     * 
     * @return {Promise}
     */
    const checkExistentInventory = inventoryRow => {
        return service.existsInventory(inventoryRow, true).then(existsRemote => {

            if (existsRemote) {

                service.showMessage('ALREADY_EXISTS_INVENTORY_REMOTE');
                return Promise.resolve(existsRemote);

            } else {

                return service.existsInventory(inventoryRow, false).then(existsLocal => {

                    if (existsLocal)
                        service.showMessage('ALREADY_EXISTS_INVENTORY_LOCAL');
                    return existsLocal;
                });
            }

        });
    };

    /**
     * Open product list of user's selected inventory
     * register.
     */
    this.openInventoryProductList = () => {
        let currentItem = templateManager.container.currentItem;
        const productListWidget = templateManager.container.getWidget('inventoryProductList');

        productListWidget.label = currentItem.DTINVENTRETI + ' - ' + currentItem.DSINVENTRETI;
        setProductListFilter(productListWidget, currentItem).then(() => {
            UIManager.goToWidget(productListWidget);
        });
    };

    /**
     * Set filter on product list according to inventory
     * selected by user on inventory lists.
     * 
     * @param {Widget} productListWidget 
     * @param {Object} selectedInventory 
     * 
     * @returns {Promise}
     */
    const setProductListFilter = (productListWidget, selectedInventory) => {
        productListWidget.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": selectedInventory.CDFILIAL
            },
            {
                "name": "NRINVENTRETI",
                "operator": "=",
                "value": selectedInventory.NRINVENTRETI
            }
        ];
        return productListWidget.reload();
    };

    /**
     * Open form to add new product related with
     * current inventory.
     */
    this.openFormToAddProduct = () => {
        const productForm = templateManager.container.getWidget('productForm');

        productForm.label = ScreenService.i18n('ADD_PRODUCT_LABEL');
        productForm.currentRow = {
            __is_new: true
        };
        productForm.getField('NMPRODUTO').isDisabled = false;

        UIManager.goToWidget(productForm);
    };

    /**
     * Open tabbed data to edit products
     * existent registers.
     */
    this.openProductEdit = () => {
        const productListWidget = templateManager.container.getWidget('inventoryProductList');
        const productEdit = templateManager.container.getWidget('productEdit');

        templateManager.container.fabVisibility = false;
        productEdit.dataSource = productListWidget.dataSource;

        UIManager.goToWidget(productEdit).then(() => {
            let index = productListWidget.dataSource.data.indexOf(templateManager.container.currentSubItem);
            productEdit.activateTab(index);
        });
    };

    /**
     * Get back from product edit widget.
     * Reload dataSource to refresh all unsaved
     * data.
     */
    this.getBackFromProductEdit = () => {
        const productListWidget = getWidget('inventoryProductList');
        productListWidget.dataSource.reload();
        setSaveButtonAcessibility(false);
        UIManager.goBack();
    };

    /**
     * Enable or disable save button.
     * 
     * @param {Boolean} enable
     */
    const setSaveButtonAcessibility = enable => {
        const productEdit = getWidget('productEdit');
        productEdit.getAction('saveProductEdit').isDisabled = !enable;
        templateManager.updateTemplate();
    };

    /**
     * Instructions to be executed when 
     * change active tab.
     * Reset save button acessibility
     * 
     * @param {Widget} productEditWidget 
     */
    this.onChangeActiveTab = productEditWidget => {
        setSaveButtonAcessibility(productEditWidget.getActiveTabData().changed);
    };

    /**
     * Instructions to be executed when change product
     * quantity. 
     * Set changed on current row and enable save button.
     */
    this.onChangeProductQuantity = () => {
        const productEditWidget = getWidget('productEdit');

        let activeData = productEditWidget.getActiveTabData();
        if (!activeData.changed) {
            activeData.changed = true;
            setSaveButtonAcessibility(true);
            activeData.tabLabel = activeData.tabLabel.replace("", "\u25CF ");
        }
    };

    /**
     * Save product register. If it's a new register
     * call function to insert new product.
     */
    this.saveProductForm = () => {
        const productForm = templateManager.container.getWidget('productForm');
        let inventory = productForm.currentRow;

        saveNewInventoryProduct(inventory);
    };

    /**
     * Save product register edited on tabbed
     * widget.
     */
    this.saveProductTabbedEdit = () => {
        const productEditWidget = getWidget('productEdit');
        saveEditedProduct(productEditWidget.getActiveTabData());
    };

    /**
     * Insert product register into related inventory
     * register. Show a sucess message and get back to
     * product list if nothing goes wrong.
     * 
     * @param {Object} product
     * 
     * @returns {Promise}
     */
    const saveNewInventoryProduct = product => {
        let currentInventory = templateManager.container.currentItem;

        return service.saveNewInventoryProduct(product, currentInventory).then(() => {
            const productListWidget = templateManager.container.getWidget('inventoryProductList');
            productListWidget.dataSource.reload();
            UIManager.goBack();
            ScreenService.notificationMessage('PRODUCT_INVENTORY_ADDED');
        });
    };

    /**
     * Save edited inventory's product register. 
     * 
     * @param {Object} product Object containing
     * product data.
     * 
     * @returns {Promise}
     */
    const saveEditedProduct = product => {
        return service.saveEditedProduct(product).then(() => {
            ScreenService.notificationMessage('PRODUCT_INVENTORY_EDITED');
            product.changed = false;
            setSaveButtonAcessibility(false);
            product.tabLabel = product.tabLabel.replace("\u25CF ", "");
        });
    };

    /**
     * Remove inventory register after user swipe 
     * card left and click 'yes' on question popup.
     * Check if is empty before remove and ask to user
     * to confirm so.
     */
    this.removeInventoryRegister = () => {
        let itemToExclude = templateManager.container.itemToExclude;

        service.checkIsInventoryEmpty(itemToExclude).then(isEmpty => {
            let remove = item => {
                service.removeInventoryRegister(item).then(() => {
                    afterRemoveInventory();
                });
            };

            if (isEmpty) {
                remove(itemToExclude);
            } else {
                ScreenService.confirmMessage('INVENTORY_HAS_PRODUCT_WANT_DELETE').then(() => {
                    remove(itemToExclude);
                });
            }
        });
    };

    /**
     * After remove inventory, show sucess notification
     * and reload datasource from inventory list widget
     * to show new set of registers.
     */
    const afterRemoveInventory = () => {
        ScreenService.notificationMessage(ScreenService.i18n('INVENTORY_REMOVED'));
        const inventoryList = templateManager.container.getWidget('manageInventory');
        inventoryList.reload();
    };

    /**
     * After remove selected inventorys, show sucess 
     * notification and reload datasource from inventory 
     * list widget to show new set of registers.
     */
    const afterRemoveSelectedInventory = () => {
        const inventoryList = templateManager.container.getWidget('manageInventory');

        ScreenService.notificationMessage(ScreenService.i18n('SELECTED_INVENTORY_REMOVED'));
        inventoryList.dataSource.clearCheckedRows();
        changeFabAction();
        inventoryList.reload();
    };

    /**
     * Remove inventory product register after user swipe 
     * card left and click 'yes' on question popup.
     */
    this.removeInventoryProductRegister = () => {
        let itemToExclude = templateManager.container.itemToExclude;

        ScreenService.confirmMessage(ScreenService.i18n('WANT_TO_REMOVE_REG')).then(() => {
            service.removeInventoryProductRegister(itemToExclude).then(() => {
                ScreenService.notificationMessage(ScreenService.i18n('PRODUCT_INVENTORY_REMOVED'));
            });
        });
    };

    /**
     * Instructions to be executed after select
     * an inventory register.
     */
    this.afterSelectInventory = () => {
        changeFabAction();
    };

    /**
     * Instructions to be executed after unselect
     * an inventory register.
     */
    this.afterUnselectInventory = () => {
        changeFabAction();
    };

    /**
     * Change fab action between add action and exclusion
     * action when user long tap and select one or more
     * registers.
     */
    const changeFabAction = () => {
        const container = templateManager.container;
        const inventoryList = container.getWidget('manageInventory');

        if (inventoryList.hasCheckedRows()) {
            inventoryList.fabAction = "ManageInventoryController.removeSelectedInventorys();";
            container.fabActionIcon = "fa-trash-alt";
        } else {
            inventoryList.fabAction = "ManageInventoryController.openFormToAddInventory();";
            container.fabActionIcon = "fa-plus";
        }

        templateManager.updateTemplate();
    };

    /**
     * Remove selected inventorys registers.
     * Check if any of the selected registers have
     * products relateds with it.
     */
    this.removeSelectedInventorys = () => {
        const inventoryList = templateManager.container.getWidget('manageInventory');

        checkSelectedInventorysAreEmpty().then(inventorysAreEmpty => {
            let msg = inventorysAreEmpty ? 'WANT_REMOVE_SELECTED_ITEMS' : 'SELECTED_INVENTORY_HAS_PRODUCT_WANT_DELETE';

            ScreenService.confirmMessage(msg).then(() => {
                service.removeSelectedInventorys(inventoryList.getCheckedRows()).then(() => {
                    afterRemoveSelectedInventory();
                });
            });
        });
    };

    /**
     * Check if any of the selected inventorys
     * to be removed have products relateds with it.
     * 
     * @returns {Promise}
     */
    const checkSelectedInventorysAreEmpty = () => {
        const inventoryList = templateManager.container.getWidget('manageInventory');
        let selectedRegisters = inventoryList.getCheckedRows();

        let checkIsInventoryEmptyPromise = [];
        selectedRegisters.forEach(inventory => {
            checkIsInventoryEmptyPromise.push(service.checkIsInventoryEmpty(inventory));
        });

        return Promise.all(checkIsInventoryEmptyPromise).then(checkedInventorys => !checkedInventorys.includes(false));
    };

    /**
     * Instructions to be executed after select
     * an inventory's product register.
     * Hide add button and show exclusion button.
     */
    this.afterSelectInventoryProduct = () => {
        setActionVisibilityOnActiveWidget('addProduct', false);
        setActionVisibilityOnActiveWidget('removeSelectedProducts', true);
    };

    /**
     * Instructions to be executed after unselect
     * an inventory's product register.
     * Check if has checked rows. If not, show add
     * button and hide remove button.
     */
    this.afterUnselectInventoryProduct = () => {
        const productListWidget = templateManager.container.getWidget('inventoryProductList');

        if (!productListWidget.hasCheckedRows()) {
            setActionVisibilityOnActiveWidget('addProduct', true);
            setActionVisibilityOnActiveWidget('removeSelectedProducts', false);
        }
    };

    /**
     * Set visibility of actions in active widget to
     * parameter value.
     * 
     * @param {String} actionName
     * @param {Boolean} visible 
     */
    const setActionVisibilityOnActiveWidget = (actionName, visible) => {
        const productListWidget = templateManager.container.getActiveWidget();
        productListWidget.getAction(actionName).isVisible = visible;
        templateManager.updateTemplate();
    };

    /**
     * Instructions to be executed when enter product
     * list widget. Set addProduct button as visible and
     * remove button as invisible.
     */
    this.onEnterInventoryProductList = () => {
        setActionVisibilityOnActiveWidget('addProduct', true);
        setActionVisibilityOnActiveWidget('removeSelectedProducts', false);
    };

    /**
     * Remove selected registers of inventory's products.
     * After remove, show a success message, reload data and
     * reset buttons visibility.
     */
    this.removeSelectedProducts = () => {
        const productListWidget = templateManager.container.getActiveWidget();

        service.removeSelectedProducts(productListWidget.getCheckedRows()).then(() => {
            ScreenService.notificationMessage('SELECTED_PRODUCT_INVENTORY_REMOVED');
            productListWidget.reload();
            this.onEnterInventoryProductList();
        });
    };

    /**
     * Copy last exported inventory productset to the current
     * one.
     */
    this.copyLastExportedInventory = () => {
        let currentItem = templateManager.container.currentItem;
        let inventoryProductList = getWidget('inventoryProductList');

        ScreenService.confirmMessage('WANT_COPY_LAST_EXP_INVT').then(() => {
            service.copyLastExportedInventory(currentItem).then(() => {
                ScreenService.successNotification('SUCCESSFUL_OPERATION');
                inventoryProductList.dataSource.reload();
            }).catch((error) => {
                ScreenService.notificationMessage(error);
            });
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ManageInventoryController', ManageInventoryController);
});
