var PurchaseRequestController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, PurchaseRequestService) {

    /**
     * @var service Reference to PurchaseRequestService.
     */
    const service = PurchaseRequestService;

    /**
     * Reference to Container's getWidget function. Return 
     * widget object by name.
     * 
     * @param {String} widgetName Name of widget to return.
     *
     * @returns {Object<Widget>} widget
     */
    const getWidget = widgetName => templateManager.container.getWidget(widgetName);

    /**
     * Check if is valid to init screen.
     * Test if have imported data, a logged 
     * unit, and check if is valid to create
     * new inventory registers.
     */
    this.initScreenIfIsValid = () => {

        if (!SchoolMealsUtil.hasImportedData()) {
            openScreenWithDashboardPushState('dataImport').then(ScreenService.showMessage('HAVE_TO_IMPORT_DATA'));
        } else if (!SchoolMealsUtil.hasLoggedUnit()) {
            openScreenWithDashboardPushState('changeUnit').then(ScreenService.showMessage('PLEASE_CHANGE_UNIT'));
        } else {
            openScreenWithDashboardPushState('purchaseRequest');
        }

    };

    /**
     * Open screen using ScreenService.openWindow and
     * push state using UIManager.pushState to correctly
     * go back to dashboard when back action is pressed.
     * 
     * @param {String} screenName Name of container to open.
     * 
     * @returns {Promise}
     */
    const openScreenWithDashboardPushState = screenName => {
        return ScreenService.openWindow(screenName).then(() => {
            UIManager.pushState(() => {
                ScreenService.openWindow('dashboard');
            });
        });
    };

    /**
     * Open form to add new purchase request
     * register.
     */
    this.openFormToAddPurchaseRequest = () => {
        let newPurchaseRequestForm = getWidget('newPurchaseRequestForm');
        newPurchaseRequestForm.currentRow = {};
        UIManager.goToWidget(newPurchaseRequestForm);
    };

    /**
     * Instructions to be executed when open form
     * to add new purchase request.
     */
    this.onEnterNewRequestForm = () => {
        let newPurchaseRequestForm = getWidget('newPurchaseRequestForm');
        filterProductListByUnitParams(newPurchaseRequestForm);
        setNewPurchaseRequestValues(newPurchaseRequestForm);
        newPurchaseRequestForm.getField('DSJUSTSOLEXT').isVisible = false;
    };

    /**
     * Filter product list by product type = "E" when unit
     * has parameter inside 'Parametros' dataSource and
     * parameters attribute 'IDUTPRODPREF' is equal to 'N'.
     * 
     * @param {Widget} newPurchaseRequestForm Form widget to 
     * add new purchase request.
     */
    const filterProductListByUnitParams = newPurchaseRequestForm => {
        let loggedUnit = getLocalVar('userData').CDFILIAL;

        service.getUnitParameters(loggedUnit).then(parameterArray => {
            let productField = newPurchaseRequestForm.getField('NMPRODUTO');
            service.setAddProductFieldFilter(productField, parameterArray);
            productField.reload();
        });
    };

    /**
     * Set default values on new purchase request register's
     * current row.
     * 
     * @param {Widget} newPurchaseRequestForm 
     */
    const setNewPurchaseRequestValues = newPurchaseRequestForm => {
        newPurchaseRequestForm.currentRow.IDITMSOLEXTRA = 'N';
        newPurchaseRequestForm.currentRow.__is_new = true;
        /**
         * @todo
         * newPurchaseRequestForm.getField('DATE').min = moment().add(1, 'days').format('L');
         */
    };

    /**
     * Instructions to be executed when user toggle
     * extra product option. Change extra justification
     * field visibility.
     * 
     * @param {Widget} purchaseRequestForm Formulary widget
     * from add or edit register.
     */
    this.onToggleExtraProduct = (purchaseRequestForm, editing) => {
        toggleExtraJustificationField(purchaseRequestForm, editing);
    };

    /**
     * Set extra justification field visibility according
     * to product extra toggle button state and set value
     * to quantity justification field's value.
     * 
     * @param {Widget} purchaseRequestForm Formulary widget
     * from add or edit register.
     */
    const toggleExtraJustificationField = (purchaseRequestForm, editing) => {
        let extraJustificationField = purchaseRequestForm.getField('DSJUSTSOLEXT');
        let currentRegister = editing ? purchaseRequestForm.getActiveTabData() : purchaseRequestForm.currentRow;

        extraJustificationField.isVisible = currentRegister.PRODUTOEXTRA;

        if (extraJustificationField.isVisible) {
            currentRegister.DSJUSTSOLEXT = currentRegister.DSJUSTOPER;
        } else {
            currentRegister.DSJUSTSOLEXT = null;
        }

        if (editing) {
            extraJustificationField.updateDescriptionText();
        }

        templateManager.updateTemplate();
    };

    /**
     * Create new product purchase request with user's data
     * input if all data is valid to proceed.
     */
    this.createNewProductPurchaseRequest = () => {
        let newPurchaseRequestForm = getWidget('newPurchaseRequestForm');

        service.createNewProductPurchaseRequest(newPurchaseRequestForm.currentRow).then(() => {

            ScreenService.notificationMessage('PRODUCT_ADDED');
            UIManager.goBack();
            getWidget('purchaseRequestList').reload();

        }).catch(errorCode => {
            ScreenService.showMessage(errorCode);
        });
    };

    /**
     * Instructions to be executed after select
     * a product request register.
     */
    this.afterSelectRequestRegister = () => {
        changeFabAction();
    };

    /**
     * Instructions to be executed after unselect
     * a product request register.
     */
    this.afterUnselectRequestRegister = () => {
        changeFabAction();
    };

    /**
     * Change fab action between add action and exclusion
     * action when user long tap and select one or more
     * registers.
     */
    const changeFabAction = () => {
        const container = templateManager.container;
        const purchaseRequestList = getWidget('purchaseRequestList');

        if (purchaseRequestList.hasCheckedRows()) {
            purchaseRequestList.fabAction = "PurchaseRequestController.removeSelectedProducts();";
            container.fabActionIcon = "fa-trash-alt";
        } else {
            purchaseRequestList.fabAction = "PurchaseRequestController.openFormToAddPurchaseRequest();";
            container.fabActionIcon = "fa-plus";
        }

        templateManager.updateTemplate();
    };

    /**
     * Remove  product register if is valid to.
     * If is not, show a message explaining what's wrong
     * with operation.
     * 
     * @param {Object} itemToExclude Object containing product
     * data to exclude.
     */
    const removeProductPurchaseRequest = itemToExclude => {
        return service.removeValidProductPurchaseRequest(itemToExclude)
            .then(() => {
                ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
                getWidget('purchaseRequestList').reload();
                return true;
            })
            .catch(errorCode => {
                ScreenService.showMessage(errorCode);
                return false;
            });
    };

    /**
     * Remove selected products purchase request
     * registers.
     */
    this.removeSelectedProducts = () => {
        const purchaseRequestList = getWidget('purchaseRequestList');

        ScreenService.confirmMessage('WANT_REMOVE_SELECTED_ITEMS').then(() => {
            service.removeSelectedProductsIfAreValid(purchaseRequestList.getCheckedRows()).then(() => {

                ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
                purchaseRequestList.reload();
                purchaseRequestList.dataSource.clearCheckedRows();
                changeFabAction();

            }).catch(errorCode => {
                ScreenService.showMessage(errorCode);
            });
        });
    };

    /**
     * Remove  swiped product register if is valid to.
     */
    this.removeSwipedProductPurchaseRequest = () => {
        let itemToExclude = templateManager.container.itemToExclude;
        removeProductPurchaseRequest(itemToExclude).then(proceed => {
            if (proceed) {
                templateManager.updateTemplate();
            }
        });
    };

    /**
     * Remove current product register if is valid to.
     */
    this.removeClickedProductPurchaseRequest = () => {
        ScreenService.confirmMessage('WANT_REMOVE_SELECTED_ITEMS').then(() => {
            removeProductPurchaseRequest(templateManager.container.currentItem).then(proceed => {
                if (proceed) {
                    UIManager.goBack();
                }
            });
        });
    };

    /**
     * Open widget to edit existent product purchase
     * request register.
     */
    this.openRequestEditFormulary = () => {
        let purchaseRequestList = getWidget('purchaseRequestList');
        let requestForm = getWidget('editPurchaseRequestForm');
        let currentProd = templateManager.container.currentItem;

        requestForm.dataSource = purchaseRequestList.dataSource;

        UIManager.goToWidget(requestForm).then(() => {
            let index = requestForm.dataSource.data.indexOf(currentProd);
            requestForm.activateTab(index);
            if (index == 0) {
                this.onChangeActiveTab();
            }
        });
    };

    /**
     * Initialize request edit form. Set Widget label,
     * currentRow and fields acessibility. Disable 
     * fields if product is invalid to edit.
     * 
     * @param {Widget} requestForm
     * @param {Object} currentProd
     * 
     * @returns {Promise}
     */
    const setupEditRequestForm = (requestForm, currentProd) => {

        return service.isValidProductToEdit(currentProd).then((isValid) => {
            let extraProdField = requestForm.getField('PRODUTOEXTRA');

            if (!currentProd._edited) {
                currentProd.PRODUTOEXTRA = !!currentProd.DSJUSTSOLEXT;
            }
            extraProdField.isDisabled = !isValid;

            let justExtraSolField = requestForm.getField('DSJUSTSOLEXT');
            justExtraSolField.isVisible = !!currentProd.DSJUSTSOLEXT;
            justExtraSolField.isDisabled = !isValid;

            requestForm.getField('QTAJUSTADA').isDisabled = !isValid;
            requestForm.getField('DSJUSTOPER').isDisabled = !isValid;

            requestForm.getAction('saveEditedProductPurchaseRequest').isDisabled = !isValid;

            templateManager.updateTemplate().then(() => {
                if (justExtraSolField.isVisible) {
                    justExtraSolField.updateDescriptionText();
                }
            });
        });
    };

    /**
     * save product purchase request edition with user's data
     * input if all data is valid to proceed.
     */
    this.saveEditedProductPurchaseRequest = () => {
        let editPurchaseRequestForm = getWidget('editPurchaseRequestForm');

        service.saveEditedProductPurchaseRequest(editPurchaseRequestForm.getActiveTabData()).then(() => {
            editPurchaseRequestForm.getActiveTabData()._edited = false;
            ScreenService.notificationMessage('PRODUCT_EDITED');
        }).catch(errorCode => {
            ScreenService.showMessage(errorCode);
        });
    };

    /**
     * On change active tab, change what fields
     * are active.
     */
    this.onChangeActiveTab = () => {
        let requestForm = getWidget('editPurchaseRequestForm');
        let currentItem = requestForm.getActiveTabData();

        setupEditRequestForm(requestForm, currentItem);
    };

    /**
     * Toggle '_edited' property into tab register, used
     * for internal control of registers setup
     * (Like in 'setupEditRequestForm'
     * function).
     *
     * @param {Boolean} state - New State
     */
    this.toggleEditedStateOnCurrentRegister = (state) => {
        let editPurchaseRequestForm = getWidget('editPurchaseRequestForm');
        editPurchaseRequestForm.getActiveTabData()._edited = state;
    };

    /**
     * Open filter popup to allow user filter request list
     * using a date period.
     */
    this.openFilterPopUp = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        dateFilterPopup.isVisible = true;
        UIManager.openPopUp(dateFilterPopup);
    };

    /**
     * Open filter popup to allow user filter request list
     * using a date period.
     */
    this.closeFilterPopUp = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        dateFilterPopup.isVisible = false;
        UIManager.goBack();
    };

    /**
     * Confirm filter request using selected date
     * by user in form.
     */
    this.filterRequestListByDate = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        const purchaseRequestList = getWidget('purchaseRequestList');
        let selectedDate = dateFilterPopup.currentRow.FILTER_DATE;

        if (!!selectedDate) {
            purchaseRequestList.dataSource.data = purchaseRequestList.dataSource._data.filter(reg => reg.DTPROGENTR == selectedDate);
        } else {
            purchaseRequestList.dataSource.data = [...purchaseRequestList.dataSource._data];
        }
        ScreenService.notificationMessage('REQUEST_FILTERED');
        this.closeFilterPopUp();
    };

    /**
     * After load dataSource, set a backup property to use
     * in filter functions.
     */
    this.afterLoadDataSourcePurReqList = () => {
        const purchaseRequestList = getWidget('purchaseRequestList');
        purchaseRequestList.dataSource._data = [...purchaseRequestList.dataSource.data];
    };

    /**
     * Clear selected date into date filter popup.
     */
    this.clearDateFilter = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        dateFilterPopup.currentRow.FILTER_DATE = null;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('PurchaseRequestController', PurchaseRequestController);
});
