var StaffManagementController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, StaffManagementService) {

    /**
     * @var service Reference to StaffManagementService.
     */
    const service = StaffManagementService;

    /**
     * Reference to Container's getWidget function. Return 
     * widget object by name.
     * 
     * @param {String} widgetName Name of widget to return.
     *
     * @returns {Object<Widget>} widget
     */
    const getWidget = widgetName => templateManager.container.getWidget(widgetName);

    /**
     * Check if is valid to init screen.
     * Test if have imported data, a logged 
     * unit, and check if is valid to create
     * new inventory registers.
     */
    this.initScreenIfIsValid = () => {

        if (!SchoolMealsUtil.hasImportedData()) {
            openScreenWithDashboardPushState('dataImport').then(ScreenService.showMessage('HAVE_TO_IMPORT_DATA'));
        } else if (!SchoolMealsUtil.hasLoggedUnit()) {
            openScreenWithDashboardPushState('changeUnit').then(ScreenService.showMessage('PLEASE_CHANGE_UNIT'));
        } else if (service.isLoggedUnitMealsCenter()) {
            ScreenService.showMessage('CANNOT_MANAGE_STAFF_IS_MEALS_CENTER');
        } else if (!service.loggedUnitHasMealsCenter()) {
            ScreenService.showMessage('CANNOT_MANAGE_STAFF_HAS_NO_MEALS_CENTER');
        } else {
            service.checkIfHasStaffInPeriod()
                .then(service.checkIfHasStaffInUnit)
                .then(() => openScreenWithDashboardPushState('staffManagement'))
                .catch(errorCode => ScreenService.showMessage(errorCode));
        }

    };

    /**
     * Open screen using ScreenService.openWindow and
     * push state using UIManager.pushState to correctly
     * go back to dashboard when back action is pressed.
     * 
     * @param {String} screenName Name of container to open.
     * 
     * @returns {Promise}
     */
    const openScreenWithDashboardPushState = screenName => {
        return ScreenService.openWindow(screenName).then(() => {
            UIManager.pushState(() => {
                ScreenService.openWindow('dashboard');
            });
        });
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service name using 
     * alphabetical order.
     * Set fab icon according to ordering direction.
     */
    this.sortServiceListByServiceName = () => {
        const staffManagement = getWidget('staffManagement');
        service.sortServiceListByServiceName(staffManagement.dataSource);
        staffManagement.container.fabActionIcon = !staffManagement.dataSource.orderingAscAlpha ? "fa-sort-alpha-up" : "fa-sort-alpha-down";
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service date.
     * Set fab icon according to ordering direction.
     */
    this.sortServiceListByDate = () => {
        const staffManagement = getWidget('staffManagement');

        service.sortServiceListByDate(staffManagement.dataSource);
        staffManagement.container.fabActionIcon = !staffManagement.dataSource.orderingAscDate ? "fa-sort-numeric-up" : "fa-sort-numeric-down";
    };

    /**
     * Open screen to edit staff registers.
     * Hide fab, get dataSource from staffListWidget
     * set default staff quantity label as '0', then
     * activate tab that contains clicked data.
     * 
     * @param {Object} clickedData 
     */
    this.openStaffEditScreen = clickedData => {
        const staffList = getWidget('staffManagement');
        const staffEditWidget = getWidget('staffEdit');

        templateManager.container.fabVisibility = false;
        staffEditWidget.dataSource = staffList.dataSource;
        staffEditWidget.getField('staffLabel').subLabel = '0';

        UIManager.goToWidget(staffEditWidget).then(() => {
            let index = staffList.dataSource.data.indexOf(clickedData);
            staffEditWidget.activateTab(index);
        });
    };

    /**
     * Instructions to be executed when change staff
     * quantity. 
     * Set changed on current row and enable save button.
     */
    this.onChangeStaffQuantity = () => {
        const staffEditWidget = getWidget('staffEdit');

        setSaveButtonAcessibility(true);

        if (!staffEditWidget.getActiveTabData().changed) {
            setStaffChangedData(staffEditWidget);
        }
        setStaffSum(staffEditWidget);
    };

    /**
     * Enable or disable save button.
     * 
     * @param {Boolean} enable
     */
    const setSaveButtonAcessibility = enable => {
        const staffEditWidget = getWidget('staffEdit');
        staffEditWidget.getAction('saveStaffEdit').isDisabled = !enable;
        templateManager.updateTemplate();
    };

    /**
     * Set changed as true in active tab data.
     * Add '●' character on changed data's label.
     * 
     * @param {Widget} staffEditWidget Staff Edit widget
     */
    const setStaffChangedData = staffEditWidget => {
        let activeData = staffEditWidget.getActiveTabData();

        activeData.changed = true;
        activeData.tabLabel = activeData.tabLabel.replace("", "\u25CF ");
    };

    /**
     * Set staff sum total in widget current tab;
     * 
     * @param {Widget} staffEditWidget Staff Edit widget
     */
    const setStaffSum = staffEditWidget => {
        let activeData = staffEditWidget.getActiveTabData();
        let sum = Number(activeData.QTCLIEFECPME) + Number(activeData.QTFUNEFECPME) + Number(activeData.QTDOAEFECPME);

        staffEditWidget.getField('staffLabel').subLabel = sum;
        activeData.QTCOMEFECPME = sum;
    };

    /**
     * Get back from staff edit widget.
     * Reload dataSource to refresh all unsaved
     * data.
     */
    this.getBackFromStaffEdit = () => {
        const staffList = getWidget('staffManagement');
        staffList.dataSource.reload();
        setSaveButtonAcessibility(false);
        UIManager.goBack();
    };

    /**
     * Save edited active staff register.
     * show success message after conclude operation.
     * Show an error message if something goes wrong.
     */
    this.saveStaffEdit = () => {
        const staffEditWidget = getWidget('staffEdit');
        let activeData = staffEditWidget.getActiveTabData();

        service.saveStaffEdit(activeData)
            .then(() => afterSaveStaffEdit(staffEditWidget))
            .then(() => ScreenService.notificationMessage('SETUP_CLIENT_QUANT'))
            .catch(errorCode => {
                ScreenService.showMessage(errorCode);
            });
    };

    /**
     * After save staff edited data, show 
     * success message and reset 'changed' property,
     * save button acessibility and unsaved data 
     * mark ('●');
     * 
     * @param {Widget} staffEditWidget
     */
    const afterSaveStaffEdit = staffEditWidget => {
        let activeData = staffEditWidget.getActiveTabData();

        ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
        activeData.changed = false;
        activeData.tabLabel = activeData.tabLabel.replace("\u25CF ", "");
        setSaveButtonAcessibility(false);
    };

    /**
     * Instructions to be executed when 
     * change active tab.
     * Reset save button acessibility
     * 
     * @param {Widget} staffEditWidget
     */
    this.onChangeActiveTab = staffEditWidget => {
        castStaffQuantitiesToNumber(staffEditWidget.getActiveTabData());
        setSaveButtonAcessibility(staffEditWidget.getActiveTabData().changed);
        setStaffSum(staffEditWidget);
    };

    /**
     * Cast quantities values to Number
     * 
     * @param {Object} staffData 
     */
    const castStaffQuantitiesToNumber = staffData => {
        staffData.QTFUNEFECPME = Number(staffData.QTFUNEFECPME);
        staffData.QTCLIEFECPME = Number(staffData.QTCLIEFECPME);
        staffData.QTCOMEFECPME = Number(staffData.QTCOMEFECPME);
        staffData.QTDOAEFECPME = Number(staffData.QTDOAEFECPME);
    };

    /**
     * Set filter on staffClientList widget dataSource
     * to bring related data. Then open widget.
     */
    this.openStaffClientList = () => {
        const staffEditWidget = getWidget('staffEdit');
        const staffClientList = getWidget('staffClientList');
        const staffList = getWidget('staffManagement');

        let staffData = staffEditWidget.getActiveTabData();

        filterStaffClientList(staffClientList, staffData);
        UIManager.goToWidget(staffClientList);
        staffEditWidget.currentActiveTabIndex = staffList.dataSource.data.indexOf(staffEditWidget.getActiveTabData());
    };

    /**
     * Build staff client list dataSourceFilter according
     * to active tab staff data.
     * 
     * @param {Widget} staffClientList 
     * @param {Object} staffData
     */
    const filterStaffClientList = (staffClientList, staffData) => {
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'CDFILIAL', staffData.CDFILIAL);
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'DTEFETCLPME', staffData.DTCARDIAPME);
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'CDSERVFILI', staffData.CDSERVFILI);
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'CDTIPOCARD', staffData.CDTIPOCARD);
        staffClientList.reload();
    };

    /**
     * Open staff edit form in popup widget.
     * Set current row as clicked item and
     * backup row as a copy of it. If user cancel operation
     * then backup is restored, else 
     */
    this.openStaffEditPopUp = () => {
        const staffClientEdit = getWidget('staffClientEdit');

        staffClientEdit.isVisible = true;
        staffClientEdit.currentRow = Object.assign({}, templateManager.container.currentItem);
        staffClientEdit.__backupRow = Object.assign({}, templateManager.container.currentItem);

        staffClientEdit.getField('staffLabel').subLabel = staffClientEdit.currentRow.TOTAL;

        UIManager.openPopUp(staffClientEdit);
    };

    /**
     * Save edited quantities of staff's client.
     * Set current item to edited current row.
     */
    this.saveStaffClientEdition = () => {
        const staffClientEdit = getWidget('staffClientEdit');

        Object.assign(templateManager.container.currentItem, staffClientEdit.currentRow);
        UIManager.goBack();
        ScreenService.notificationMessage('QUANTITY_EDITED_SUCCESFUL');
    };

    /**
     * Cancel edited quantities of staff's client.
     * Reset current item to previous object.
     */
    this.cancelStaffClientEdition = () => {
        const staffClientEdit = getWidget('staffClientEdit');

        staffClientEdit.currentRow = Object.assign({}, staffClientEdit.__backupRow);
        UIManager.goBack();
    };

    /**
     * Instructions to be executed when change staff client
     * quantity. 
     * Set changed on current row and enable save button.
     */
    this.onChangeStaffClientQuantity = () => {
        const staffClientEdit = getWidget('staffClientEdit');
        castStaffClientQuantitiesToNumber(staffClientEdit.currentRow);
        setStaffClientSum(staffClientEdit);
    };

    /**
     * Cast client quantities values to Number
     * 
     * @param {Object} staffClientData
     */
    const castStaffClientQuantitiesToNumber = staffClientData => {
        staffClientData.QTCOMEFCLPME = Number(staffClientData.QTCOMEFCLPME);
        staffClientData.QTDOAEFCLPME = Number(staffClientData.QTDOAEFCLPME);
        staffClientData.QTFUNEFCLPME = Number(staffClientData.QTFUNEFCLPME);
    };

    /**
     * Set staff client sum total in popup widget;
     * 
     * @param {Widget} staffClientWidget Staff client Edit widget
     */
    const setStaffClientSum = staffClientWidget => {
        let currentRow = staffClientWidget.currentRow;
        let sum = Number(currentRow.QTDOAEFCLPME) + Number(currentRow.QTFUNEFCLPME) + Number(currentRow.QTCOMEFCLPME);

        staffClientWidget.getField('staffLabel').subLabel = sum;
        currentRow.TOTAL = sum;
    };

    /**
     * Save all staff client quantities on 
     * 'ClientesDoServicoEfetivos' dataSource if
     * is valid.
     * Client data is valid if sum of all client's
     * quantities is equal to service total staff
     * quantity.
     * 
     */
    this.saveStaffClientQuantities = () => {
        const staffClientList = getWidget('staffClientList');
        const staffEditWidget = getWidget('staffEdit');

        let staffClientListData = staffClientList.dataSource.data;
        let staffRegister = staffEditWidget.dataSource.data[staffEditWidget.currentActiveTabIndex];

        service.saveStaffClientQuantities(staffClientListData, staffRegister).then(() => {
            UIManager.goBack();

            ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
            let activeDataIndex = staffEditWidget.dataSource.data.indexOf(staffEditWidget.getActiveTabData());
            staffEditWidget.activateTab(activeDataIndex);
            staffEditWidget.updateHeaderScroll();
        }).catch(errorCode => {
            ScreenService.showMessage(errorCode);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('StaffManagementController', StaffManagementController);
});
