var DataExportService = function(restEngine, SchoolMealsUtil, ScreenService, UIManager) {

    /**
     * Check if current user is active on system.
     * If it's not, so should logout user.
     * 
     * @return {Promise}
     */
    this.validateActiveUser = () => {
        let userMail = getLocalVar('userData').DSEMAILOPER;
        let nrorg = getLocalVar('userData').NRORG;

        return checkActiveUser(userMail, nrorg).then(isActive => {
            if (isActive) {
                return Promise.resolve();
            } else {
                return Promise.reject();
            }
        });
    };

    /**
     * Check if user is active.
     * 
     * @param {String} mail User mail.
     * @param {String|Number} nrorg
     * 
     * @return {Promise}
     */
    const checkActiveUser = function(mail, nrorg) {
        return restEngine.request('/UsuarioAtivo', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "DSEMAILOPER",
                    "operator": "=",
                    "value": mail
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                }
            ]
        }, null, null, null, null, 300000).then(function(result) {
            return result.dataset.usuario.ativo;
        });
    };

    /**
     * Bind inventory's products to inventory register.
     * Set product into inventory object and save into
     * indexed db.
     * 
     * @return {Promise} 
     */
    this.bindProductListToInventory = () => {
        let promises = [
            SchoolMealsUtil.getIDBVar('ProdutosInventario'),
            SchoolMealsUtil.getIDBVar('Inventarios')
        ];

        return Promise.all(promises).then(result => {
            let productInventory = result[0];
            let inventoryRegisters = result[1];

            inventoryRegisters.forEach(inventory => {
                inventory.PRODUTOS = productInventory.filter(product =>
                    product.NRINVENTRETI === inventory.NRINVENTRETI &&
                    product.CDFILIAL === inventory.CDFILIAL
                );
            });

            return SchoolMealsUtil.setIDBVar('Inventarios', inventoryRegisters);
        });
    };

    /**
     * Set on deleted products datasource only products
     * from requests that exists in remote server.
     * 
     * @returns {Promise}
     */
    this.removeDeletedProductsFromInexistentRequest = () => {
        return SchoolMealsUtil.getIDBVar('ProdutosExcluidos').then(deletedProducts => {
            deletedProducts = deletedProducts.filter(product => !!product.NRSOLICMP);
            return SchoolMealsUtil.setIDBVar('ProdutosExcluidos', deletedProducts);
        });
    };

    /**
     * Set on deleted products datasource only products
     * from requests that exists in remote server.
     * 
     * @returns {Promise}
     */
    this.removeEditedProductsFromInexistentRequest = () => {
        return SchoolMealsUtil.getIDBVar('ProdutosSolicitacao').then(editedProducts => {
            editedProducts = editedProducts.filter(product => !!product.NRSOLICMP && product.edited);
            return SchoolMealsUtil.setIDBVar('ProdutosEditados', editedProducts);
        });
    };

    /**
     * Set on MOVOPSOL datasource only registers
     * from requests that exists in remote server.
     * 
     * @returns {Promise}
     */
    this.removeLogRegisterFromInexistentRequest = () => {
        return SchoolMealsUtil.getIDBVar('Movopsol').then(logRegisterList => {
            logRegisterList = logRegisterList.filter(logMov => !!logMov.NRSOLIMOV);
            return SchoolMealsUtil.setIDBVar('Movopsol', logRegisterList);
        });
    };

    /**
     * Update request list and set extra request registers
     * according to extra products added to indexed db
     * datasource.
     */
    this.updateRequestList = () => {
        let promises = [
            SchoolMealsUtil.getIDBVar('ProdutosSolicitacao'),
            SchoolMealsUtil.getIDBVar('NovaSolicitacao'),
            SchoolMealsUtil.getIDBVar('Movopsol')
        ];

        return Promise.all(promises)
            .then(result => Object({
                requestProducts: result[0],
                newRequest: result[1],
                movopsol_log: result[2]
            }))
            .then(updateExtrasRequest)
            .then(updateRequestData)
            .then(setRequestDataSource);
    };

    /**
     * Update products and logs that are related
     * with each request register. Set requests data
     * to be exported.
     * 
     * @param {Object} dataObject Object containing
     * data from 'ProdutosSolicitacao', 'NovaSolicitacao'
     * and 'Movopsol' dataSources.
     * 
     * 
     */
    const updateRequestData = dataObject => {
        dataObject.newRequest.forEach(request => {
            let products = getProductsByRequest(request, dataObject.requestProducts);

            if (products.length) {
                let logs = getLogsByRequest(request, dataObject.movopsol_log);

                request.QTITEMSOLIC = products.length;
                request.PRODUTOS = products;
                request.LOG = logs;
            }

            // request.DTSOLCMP = moment(request.DTSOLCMP).format('DD/MM/YYYY');
        });

        dataObject.newRequest = dataObject.newRequest.filter(request => request.QTITEMSOLIC);

        return Promise.resolve(dataObject);
    };

    /**
     * Update products and logs that are related
     * with each request register.
     * 
     * @param {Object} dataObject Object containing
     * data from 'ProdutosSolicitacao', 'NovaSolicitacao'
     * and 'Movopsol' dataSources.
     * 
     */
    const updateExtrasRequest = dataObject => {
        let extraRequests = [];

        dataObject.newRequest.forEach(request => {
            let products = getProductsByExtraRequest(request, dataObject.requestProducts);

            if (products.length) {
                let logs = getLogsByExtraRequest(request, dataObject.movopsol_log);

                // DTSOLCMP: moment(request.DTSOLCMP).format('DD/MM/YYYY'),
                extraRequests.push(Object.assign({}, request, {
                    IDSOLEXTRA: "S",
                    QTITEMSOLIC: products.length,
                    PRODUTOS: products,
                    LOG: logs
                }));
            }
        });

        dataObject.newRequest = dataObject.newRequest.concat(extraRequests);

        return Promise.resolve(dataObject);
    };

    /**
     * Return product that are related with
     * request register and are not from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestProducts 
     */
    const getProductsByRequest = (request, requestProducts) => {
        let filteredProducts = requestProducts.filter(product =>
            product.CDFILSOLI == request.CDFILSOLI &&
            !product.NRSOLICMP &&
            product.IDITMSOLEXTRA == 'N' &&
            product.QTSOLIPROD > 0 &&
            product.inserted
        );

        filteredProducts.forEach(product => {
            product.DTPROGENTR = moment(product.DTPROGENTR).format('DD/MM/YYYY');
            product.DTUTILIZA = moment(product.DTUTILIZA).format('DD/MM/YYYY');
        });

        return filteredProducts;
    };

    /**
     * Return product that are related with
     * request register and are from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestProducts 
     */
    const getProductsByExtraRequest = (request, requestProducts) => {
        let filteredProducts = requestProducts.filter(product =>
            product.CDFILSOLI == request.CDFILSOLI &&
            !product.NRSOLICMP &&
            product.IDITMSOLEXTRA == 'S' &&
            product.inserted
        );

        filteredProducts.forEach(product => {
            product.DTPROGENTR = moment(product.DTPROGENTR).format('DD/MM/YYYY');
            product.DTUTILIZA = moment(product.DTUTILIZA).format('DD/MM/YYYY');
        });

        return filteredProducts;
    };

    /**
     * Return log registers that are related with
     * request register and are not from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestLogs 
     */
    const getLogsByRequest = (request, requestLogs) => {
        let filteredLog = requestLogs.filter(log =>
            log.CDFILSOLMOV == request.CDFILSOLI &&
            log.IDITMSOLEXTRA == "N" &&
            !log.NRSOLIMOV
        );

        filteredLog.forEach(log => {
            log.DTPROGENTR = moment(log.DTPROGENTR).format('DD/MM/YYYY');
        });

        return filteredLog;
    };

    /**
     * Return log registers that are related with
     * request register and are from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestLogs 
     */
    const getLogsByExtraRequest = (request, requestLogs) => {
        let filteredLog = requestLogs.filter(log =>
            log.CDFILSOLMOV == request.CDFILSOLI &&
            log.IDITMSOLEXTRA == "S" &&
            !log.NRSOLIMOV
        );

        filteredLog.forEach(log => {
            log.DTPROGENTR = moment(log.DTPROGENTR).format('DD/MM/YYYY');
        });

        return filteredLog;
    };

    /**
     * Set updated request registers into correct
     * datasource.
     * 
     * @param {Object} dataObject Object containing
     * data from 'ProdutosSolicitacao', 'NovaSolicitacao'
     * and 'Movopsol' dataSources.
     * 
     * @returns {Promise}
     */
    const setRequestDataSource = dataObject => SchoolMealsUtil.setIDBVar('NovaSolicitacao', dataObject.newRequest);

    /**
     * Save local dataSource chages and insertions
     * to remote server.
     * 
     * @param {String} dataSourceName 
     * 
     * @returns {Promise}
     */
    this.saveLocalDataSourceToRemote = dataSourceName => {
        return getDataToExport(dataSourceName)
            .then(dataToExport => saveRemoteData(dataSourceName, dataToExport));
    };

    /**
     * Save local dataSource to remote.
     * 
     * @param {String} dataSourceName
     * @param {Array} data 
     * 
     * @returns {Promise}
     */
    const saveRemoteData = (dataSourceName, data) => {
        if (data.length) {
            let saveRequestUrl = `/${dataSourceName}/save`;
            return restEngine.request(saveRequestUrl, {
                "requestType": "DataSet",
                "dataset": data
            });
        } else {
            return Promise.resolve();
        }
    };

    /**
     * Get edited and inserted data existent
     * in dataSource to export to remote storage.
     * 
     * @param {String} dataSourceName 
     * 
     * @returns {Promise}
     */
    const getDataToExport = dataSourceName => {
        return SchoolMealsUtil.getIDBVar(dataSourceName).then(dataSet => (dataSet && dataSet.filter(data => data.edited || data.inserted)) || []);
    };

    /**
     * Set locally dataSource's value to
     * empty array.
     * 
     * @param {String} dataSourceName
     * 
     * @returns {Promise}
     */
    const clearDataSource = dataSourceName => SchoolMealsUtil.setIDBVar(dataSourceName, []);

    /**
     * Export all datasources from local to remote 
     * storage by doing a request and save data.
     * After export, set all dataSources as empty arrays.
     * 
     * @return {Promise} 
     */
    this.exportAllDataSources = () => {
        let exportPromiseArray = [];
        let dataSourcesToExport = ['NovaSolicitacao', 'ProdutosEditados', 'ProdutosExcluidos', 'Inventarios',
            'Movopsol', 'ServicosCardapioEfetivos', 'ClientesDoServicoEfetivos'
        ];

        dataSourcesToExport.forEach(dataSourceName => {
            exportPromiseArray.push(
                this.saveLocalDataSourceToRemote(dataSourceName)
                .then(() => clearDataSource(dataSourceName))
            );
        });

        return Promise.all(exportPromiseArray);
    };

    /**
     * After export dataSources, import again secondary
     * datasources using same paramters used in importation
     * routine.
     * 
     * @param {DataImportService} DataImportService 
     * 
     * @returns {Promise}
     */
    this.afterExportDataSource = (exportResultDataset, DataImportService) => SchoolMealsUtil.executePromisesSequentialy([
        removeSecondaryDataSources,
        () => refreshSecondaryDataSources(DataImportService),
        updateRemoteUserData,
        updateLocalRequestProducts,
        () => finalizeExportation(exportResultDataset, DataImportService)
    ]);

    /**
     * Remove unexported secondary dataSources, to export
     * them again later.
     * 
     * @returns {Promise}
     */
    const removeSecondaryDataSources = () => {
        let promises = [];
        let dataSourcesToRemove = [
            "DatasInventario",
            "UltimoInventario",
            "ProdutosSolicitacao",
            "Solicitacao",
            "ProximoCodigoSolicitacao",
            "Parametros"
        ];

        dataSourcesToRemove.forEach(dataSourceName => {
            promises.push(
                clearDataSource(dataSourceName)
            );
        });

        return Promise.all(promises);
    };

    /**
     * Refresh secondary dataSources, get all data
     * using import parameters.
     * 
     * @param {DataImportService<Prototype>} DataImportService
     * Import data service.
     * 
     * @returns {Promise}
     */
    const refreshSecondaryDataSources = DataImportService => {
        return getParametersToRefreshData().then(params => Promise.all([
            DataImportService.getLastInventory(params),
            DataImportService.getInventoryDates(params),
            DataImportService.getSystemParameters(params),
            DataImportService.getRequestDeliveryDate(params),
            DataImportService.getRequestsList(params),
            DataImportService.getNextCodeRequest(params)
        ]));
    };

    /** 
     * Return parameters to request data from
     * secondary dataSources.
     * 
     * @returns {Promise}
     */
    const getParametersToRefreshData = () => getLastRequestDeliveryDate().then(lastRequestDeliveryDate =>
        SchoolMealsUtil.getIDBVar('availableUnitsToChange').then(availableUnitsToChange => {
            let userData = getLocalVar('userData');

            return Object.assign(userData, {
                mappedRelatedUnits: availableUnitsToChange.map(unit => unit.CDFILIAL),
                INTERVAL_START: userData.INTERVAL_START,
                INTERVAL_END: lastRequestDeliveryDate
            });
        }));

    /**
     * Return most recent request delivery date.
     * 
     * @returns {Promise}
     */
    const getLastRequestDeliveryDate = () => SchoolMealsUtil.getIDBVar('DatasEntregaSolicitacao')
        .then(requestDeliveryDates => requestDeliveryDates.map(reg => reg.DATA).reduce((date1, date2) => {
            return moment(date1) > moment(date2) ? date1 : date2;
        }, getLocalVar('userData').INTERVAL_END));

    /**
     * Update remote user data. 
     * Set unit code, user code and organization
     * number.
     * 
     * @returns {Promise}
     */
    const updateRemoteUserData = () => {
        let userData = getLocalVar('userData');

        return restEngine.request('/User/save', {
            "requestType": "Row",
            "row": {
                "NRORG": userData.NRORG,
                "CDOPERADOR": userData.CDOPERADOR,
                "CDFILIAL": userData.CDFILIAL
            }
        });
    };

    /**
     * Refresh local request products data of
     * all parametrized unit code.
     */
    const updateLocalRequestProducts = () => {
        return SchoolMealsUtil.executePromisesSequentialy([
            getAvailableUnitsToChange,
            getImportedUnitsRequestProducts
        ]);
    };

    /**
     * Get available units to change from indexed
     * db local storage.
     * 
     * @returns {Promise}
     */
    const getAvailableUnitsToChange = () =>
        SchoolMealsUtil.getIDBVar('availableUnitsToChange')
        .then(units => units.map(unit => unit.CDFILIAL));

    /**
     * Returns purchasing request products
     * according to unit.
     * 
     * @param {Array} unitArray Array of unit's
     * codes.
     * 
     * @returns {Promise}
     */
    const getImportedUnitsRequestProducts = unitArray => {
        let promises = [];

        unitArray.forEach(unit => {
            promises.push(getRequestProductsByUnit(unit));
        });

        return Promise.all(promises)
            .then(data => SchoolMealsUtil.setIDBVar('ProdutosSolicitacao', data.flat()));
    };

    /**
     * Get request products related with unit
     * register.
     * 
     * @param {String} unitCode 
     */
    const getRequestProductsByUnit = unitCode =>
        getParametersToRefreshData().then((params) => {
            return SchoolMealsUtil.doFilterDataRequest(
                "/ProdutosSolicitacao", {
                    filial: unitCode,
                    nrorg: params.NRORG,
                    dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
                    dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
                    periodo: 0
                }
            );
        });

    /**
     * Finalize exportation proccess.
     * Show success message according to approved
     * request's products.
     * 
     * @param {Array} exportResultDataset 
     * @param {DataImportService} DataImportService 
     */
    const finalizeExportation = (exportResultDataset, DataImportService) => {
        let approvedProducts = exportResultDataset[1].concat(exportResultDataset[2]);
        if (approvedProducts.length) {
            buildRequestLog(approvedProducts).then(() => {
                showSuccesLogMessage();
            });
        } else {
            ScreenService.showMessage('SUCCESSFUL_EXPORTATION');
        }
    };

    /**
     * Show custom message to bring user the
     * option to get to the log page and check 
     * what services are not changed.
     */
    const showSuccesLogMessage = () => {
        ScreenService.showCustomDialog('SUCCESSFUL_EXPORTATION_WITH_NO_PROD_CHANGES',
            'SUCCESS', 'exclamation', [{
                    label: "OK",
                    code: () => {}
                },
                {
                    label: "Visualizar",
                    code: () => {}
                }
            ]
        );
    };

    /**
     * Build request logs after export all
     * data.
     * 
     * @param {Array} approvedProducts 
     * 
     * @returns {Promise}
     */
    const buildRequestLog = approvedProducts => {
        return Util.getIDBVar('LogSolicitacao').then(requestLog => {
            if (!requestLog) {
                requestLog = [];
            }

            let currentDate = moment();
            let objLog = {
                "ID": "SM" + (requestLog.length + 1),
                "DATA": currentDate.format('DD/MM/YYYY'),
                "HORA": currentDate.format('H:mm:ss'),
                "DATAHORA": currentDate.format('DD/MM/YYYY - H:mm:ss'),
                "STATUS": "ERROR"
            };

            requestLog.pop();
            requestLog.unshift(objLog);

            return Util.setIDBVar('LogSolicitacao', requestLog).then(() => buildRequetsProductsLog(approvedProducts, objLog.ID));

        });
    };

    /**
     * Save products logs related with request register
     * that call this function.
     * 
     * @param {Array} approvedProducts 
     * @param {String} parentId
     * 
     * @returns {Promise}
     */
    const buildRequetsProductsLog = (approvedProducts, parentId) => {
        return Util.getIDBVar('LogProdutosSolicitacao').then(productRequestLog => {
            approvedProducts.forEach((product, index) => {
                product.ID = 'SM' + (index + 1);
                product.PARENT = parentId;
            });

            productRequestLog.push(approvedProducts);

            return Util.setIDBVar('LogProdutosSolicitacao', productRequestLog);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataExportService', DataExportService);
});
