var DataImportService = function(restEngine, SchoolMealsUtil) {

    /**
     * Clear all imported data from app.
     */
    this.clearImportData = () => {
        const importedDataToClean = [
            "AF", "CardapioEfetivos", "ClientesDoServicoEfetivos",
            "DatasCardapioEfetivos", "DatasEntregaSolicitacao", "DatasInventario",
            "Justificativas", "Parametros", "ProximoCodigoInventario", "ProximoCodigoProdutoInventario",
            "ServicosCardapioEfetivos", "Solicitacao", "UltimoInventario",
            "availableUnitsToChange", "reducedUniqueRequestedProducts", "requestedProducts"
        ];

        let promiseQueue = [];
        importedDataToClean.forEach(idbIndex => {
            promiseQueue.push(SchoolMealsUtil.setIDBVar(idbIndex, null));
        });

        return Promise.all(promiseQueue);
    };

    /**
     * Import requested products data.
     * 
     * @param {Object} params Data search parameters
     * @param {Array} units Selected units
     * 
     * @return {Promise}
     */
    this.importRequestProductsData = (params, units) => {
        let requestQueue = [];

        units.forEach(unit => {
            params.unit = unit.CDFILIAL;
            requestQueue.push(this.getRequestedProducts(params));
        });

        return Promise.all(requestQueue).then(allData => allData.flat());
    };

    /**
     * Return units that are relateds with current
     * user.
     * 
     * @param {String} cdoperador User code.
     * @param {Array<String>} filiais Units codes.
     * 
     * @return {Promise}
     */
    this.getUserRelatedUnits = (cdoperador, filiais) => restEngine.request('/FiliaisOperador', {
        "requestType": "Row",
        "row": {
            "CDOPERADOR": cdoperador,
            "CDFILIAIS": filiais
        }
    }, null, null, null, null, 300000).then(function(result) {
        return result.dataset.Units;
    });

    /**
     * Import products data.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.importProductsData = params => {
        return this.getProductAmount(params).then(productAmount => {
            let requestParams = {
                nrorg: getLocalVar('userData').NRORG,
                cdgrupprod: params.CDGRUPPROD || "T",
                initialAmount: 1
            };

            let requestQueue = [];

            params.relatedUnits.forEach(unit => {
                requestParams.unit = unit.CDFILIAL;
                requestParams.finalAmount = productAmount[unit.CDFILIAL];

                requestQueue.push(this.getProducts(requestParams));
            });

            return Promise.all(requestQueue).then(allData => allData.flat());
        });
    };

    /**
     * Import general system data after import all products.
     * 
     * @param {Object} params
     * 
     * @return {Promise}
     */
    this.importGeneralData = params => Promise.all([
        this.getSystemParameters(params),
        this.getJustifications(params),
        this.getRequestDeliveryDate(params),
        this.getRequestsList(params),
        this.getMenusDates(params),
        this.getMenusTypes(params),
        this.getMenusServices(params),
        this.getMenusClients(params),
        this.getMenusList(params),
        this.getLastInventory(params),
        this.getNextInventoryCode(params),
        this.getNextInventoryProductCode(params),
        this.getInventoryDates(params),
        this.getSupplyAuthorization(params)
    ]).then(allGeneralData => {
        let generalDataObject = {};

        allGeneralData.forEach(dataObject => {
            generalDataObject = Object.assign(generalDataObject, dataObject);
        });

        return generalDataObject;
    });

    /**
     * 
     * ALL DATA FILTER FUNCTIONS
     * USED BY AHEAD FUNCTIONS
     * 
     */

    /**
     * Return requested products data.
     * 
     * @param {Object} params Parameters to filter
     * data result.
     * 
     * @return {Promise}
     */
    this.getRequestedProducts = (params) => SchoolMealsUtil.doFilterDataRequest('/ProdutosSolicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filial: params.unit,
        dataInicial: params.start,
        dataFinal: params.end,
        periodo: params.period
    }).then(dataset => dataset.ProdutosSolicitacao);

    /**
     * Get products total amount.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getProductAmount = params => SchoolMealsUtil.doFilterDataRequest('/QtdTotalProdutos', {
        NRORG: getLocalVar('userData').NRORG,
        CDFILIAL: params.mappedRelatedUnits,
        CDGRUPPROD: params.CDGRUPPROD || "T"
    }).then(dataset => dataset.qtdTotalProdutos);

    /**
     * Get products data.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getProducts = params => SchoolMealsUtil.doFilterDataRequest('/Produtos', {
        nrorg: getLocalVar('userData').NRORG,
        filiais: params.unit,
        cdgrupprod: params.cdgrupprod,
        qtdInicial: params.initialAmount,
        qtdFinal: params.finalAmount
    }).then(dataset => dataset.Produtos);

    /**
     * Get system's parameters data.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getSystemParameters = params => SchoolMealsUtil.doFilterDataRequest('/Parametros', {
        cdfilial: params.mappedRelatedUnits,
        nrorg: getLocalVar('userData').NRORG
    });

    /**
     * Get system's default justifications for
     * changes in process registers.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getJustifications = params => SchoolMealsUtil.doFilterDataRequest('/Justificativas', {
        nrorg: getLocalVar('userData').NRORG
    });

    /**
     * Get system's request delivery date.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getRequestDeliveryDate = params => SchoolMealsUtil.doFilterDataRequest('/DatasEntregaSolicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get system's requests registers.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getRequestsList = params => SchoolMealsUtil.doFilterDataRequest('/Solicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period,
        userMail: getLocalVar('userData').DSEMAILOPER
    });

    /**
     * Get all menus dates according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusDates = params => SchoolMealsUtil.doFilterDataRequest('/DatasCardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get all menus types according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusTypes = params => SchoolMealsUtil.doFilterDataRequest('/TiposCardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get all services types according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusServices = params => SchoolMealsUtil.doFilterDataRequest('/ServicosCardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get all Clients types according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusClients = params => SchoolMealsUtil.doFilterDataRequest('/ClientesDoServicoEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get menu list.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusList = params => SchoolMealsUtil.doFilterDataRequest('/CardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get last inventory register according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getLastInventory = params => SchoolMealsUtil.doFilterDataRequest('/UltimoInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get next inventory code according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getNextInventoryCode = params => SchoolMealsUtil.doFilterDataRequest('/ProximoCodigoInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get next inventory product code according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getNextInventoryProductCode = params => SchoolMealsUtil.doFilterDataRequest('/ProximoCodigoProdutoInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get inventory dates according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getInventoryDates = params => SchoolMealsUtil.doFilterDataRequest('/DatasInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get next purchasing request registers code.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getNextCodeRequest = params => SchoolMealsUtil.doFilterDataRequest('/ProximoCodigoSolicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get supply authorization registers
     * 
     * @param {String} cdoperador User code.
     * @param {Array<String>} filiais Units codes.
     * 
     * @return {Promise}
     */
    this.getSupplyAuthorization = params => restEngine.request('/AF', {
        "requestType": "Row",
        "row": {
            "NRORG": getLocalVar('userData').NRORG,
            "FILIAIS": params.mappedRelatedUnits,
            "DATAINICIAL": moment(params.INTERVAL_START).format('DD/MM/YYYY'),
            "DATAFINAL": moment(params.INTERVAL_END).format('DD/MM/YYYY'),
            "PERIODO": params.period
        }
    }, null, null, null, null, 300000).then(function(result) {
        return result.dataset;
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataImportService', DataImportService);
});
