var ManageInventoryService = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, ZhUtil) {
    const util = SchoolMealsUtil;

    /**
     * Show message according to words file code, using i18n 
     * function.
     * 
     * @param {String} messageCode Message code according to
     * words file.
     */
    this.showMessage = messageCode => ScreenService.showMessage(ScreenService.i18n(messageCode));

    /**
     * Check if current unit is a school meals center.
     * 
     * @return {Boolean}
     */
    this.isCurrentUnitMealsCenter = () => {
        let userData = getLocalVar('userData');
        return userData.CDFILIAL == userData.CDFILCENTPME;
    };

    /**
     * Return if working unit have a meals center
     * (CDFILCENTPME).
     * 
     * @return {String} Meals center unit code.
     */
    this.currentUnitHaveMealsCenter = () => {
        return getLocalVar('userData').CDFILCENTPME;
    };

    /**
     * Check if all required fields are filled
     * when adding a new inventory register.
     * 
     * @param {Object} inventoryRow Object containing
     * inventory data.
     */
    this.isInventoryRequiredFieldsFilled = inventoryRow => {
        return inventoryRow.DATE && inventoryRow.DSINVENTRETI;
    };

    /**
     * Check if inventory date is less than or equal to
     * importation date.
     * 
     * @param {Object} inventoryRow Object containing
     * inventory data.
     */
    this.isInventoryDateValid = inventoryRow => {
        return moment(inventoryRow.DATE) <= moment(getLocalVar('userData').INTERVAL_END);
    };

    /**
     * Check if already exists an inventory register by checking
     * inventory date and related unit.
     * If remote is setted to true, checks exported
     * dataset, else checks at local dataset.
     * 
     * @param {Object} inventoryRow Object containing
     * inventory data.
     * 
     * @return {Promise}
     */
    this.existsInventory = (inventoryRow, remote = false) => {
        let idbKey = remote ? 'DatasInventario' : 'Inventarios';
        let loggedUnit = getLocalVar('CDFILIAL');

        return util.getIDBVar(idbKey).then(inventoryList => {
            return inventoryList && !!inventoryList.find(inventory =>
                moment(inventory.DTINVENTRETI, "DD/MM/YYYY").isSame(moment(inventoryRow.DATE)) &&
                inventory.CDFILIAL == loggedUnit
            );
        });
    };

    /**
     * Insert a new inventory register in related indexed db.
     * If not exists IDB key for inventory datasource, create
     * an empty datasource to save inventory registers.
     * 
     * @param {Object} inventoryRow Object containing inventory
     * data.
     */
    this.insertInventoryRegister = inventoryRow => {
        setupInventoryRegisterToInsert(inventoryRow).then(() => {

            util.insertIntoDataSource('Inventarios', inventoryRow).then(() => {

                setInventoryIsEmptyObject(inventoryRow, true).then(() => {
                    afterInsertInventoryRegister(inventoryRow);
                });

            }).catch(errorCode => {

                if (errorCode == 'IDB_KEY_NOT_EXISTS') {
                    errorHandlerInventoryInsertion(inventoryRow);
                }

            });

        });
    };

    /**
     * Set object in indexed db used to check if a inventory
     * is empty. Implemented instead of a loop verification of
     * all products and inventorys registers because have a lower
     * processing cost.
     * 
     * @param {object} inventory
     * @param {Boolean} empty
     * 
     * @returns {Promise}
     */
    const setInventoryIsEmptyObject = (inventory, empty) => {
        return util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmptyObject => {
            let toSaveObject = {};

            if (inventoryIsEmptyObject) {
                toSaveObject = inventoryIsEmptyObject;
            }

            let objKey = inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
            toSaveObject[objKey] = empty;

            return util.setIDBVar('inventoryIsEmpty', toSaveObject);
        });
    };

    /**
     * Set inventory row properties to save correctly.
     * Set unit code, user code, inventory's date and description,
     * organization number and generate a new code to inventory.
     * 
     * @param {Object} inventoryRow Object containing inventory
     * data.
     * 
     * @returns {Promise}
     */
    const setupInventoryRegisterToInsert = inventoryRow => {
        const userData = getLocalVar('userData');

        inventoryRow.CDFILIAL = userData.CDFILIAL;
        inventoryRow.DTINVENTRETI = moment(inventoryRow.DATE).format("DD/MM/YYYY");
        inventoryRow.CDOPERINCLUSAO = userData.CDOPERADOR;
        inventoryRow.NRORG = userData.NRORGTRAB;
        inventoryRow.inserted = true;
        return generateInventoryCode().then(nextCode => {
            inventoryRow.NRINVENTRETI = nextCode;
        });
    };

    /**
     * Generate new code to use in a inventory register insertion.
     * Generate according to inventory unit and date.
     * 
     * @returns {Promise}
     */
    const generateInventoryCode = () => {
        let unitCode = getLocalVar('userData').CDFILIAL;

        return util.getIDBVar('ProximoCodigoInventario').then(nextCodeDataSource => {
            let nextCode = false;

            nextCodeDataSource.forEach(nextCodeReg => {
                if (nextCodeReg.CDFILIAL == unitCode) {
                    nextCode = nextCodeReg.PROXIMOCODIGO;
                    nextCodeReg.PROXIMOCODIGO = String(Number(nextCodeReg.PROXIMOCODIGO) + 1).padStart(10, '0');
                }
            });

            if (nextCode) {
                util.setIDBVar('ProximoCodigoInventario', nextCodeDataSource);
            }

            return nextCode;
        });
    };

    /**
     * Get back to previous list and show a success notification
     * after insert a new inventory register.
     * 
     * @param {Object} inventoryRow Object containing inventory
     * data.
     */
    const afterInsertInventoryRegister = inventoryRow => {
        setNextInventoryProductCode(inventoryRow.NRINVENTRETI);
        UIManager.goBack();
        templateManager.container.getActiveWidget().reload();
        ScreenService.notificationMessage(ScreenService.i18n('INVENTORY_ADDED'));
    };

    /**
     * Set next inventory's product code.
     * 
     * @param {String} inventoryCode 
     */
    const setNextInventoryProductCode = inventoryCode => {
        util.getIDBVar('nextInventoryProductCode').then(nextInventoryProductCode => {
            let code = '0001';

            if (nextInventoryProductCode[inventoryCode] !== undefined) {
                code = String(Number(nextInventoryProductCode[inventoryCode]) + 1).padStart(4, '0');
            }

            nextInventoryProductCode[inventoryCode] = code;

            util.setIDBVar('nextInventoryProductCode', nextInventoryProductCode);
        });
    };

    /**
     * Handle error after try to insert a inventory register.
     * Create an IDB register to store inventory list, then
     * try to insert it again.
     * 
     * @param {Object} inventoryRow
     */
    const errorHandlerInventoryInsertion = (inventoryRow) => {
        util.setIDBVar('Inventarios', []).then(() => {
            util.insertIntoDataSource('Inventarios', inventoryRow).then(() => {
                setInventoryIsEmptyObject(inventoryRow, true).then(() => {
                    afterInsertInventoryRegister(inventoryRow);
                });
            });
        });
    };

    /**
     * Save new product register related with selected inventory.
     * Check if register is valid and if already exists before 
     * try to insert.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing inventory
     * data.
     * 
     * @returns {Promise}
     */
    this.saveNewInventoryProduct = (product, inventory) => {
        return checkProductRowValidity(product, inventory)
            .then(() => insertNewProduct(product, inventory))
            .catch(errorCode => this.showMessage(errorCode));
    };

    /**
     * Check if product register is valid to insert.
     * Check if all fields are filled, if product already
     * exists in inventory and if quantity is greater
     * than zero.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise}
     */
    const checkProductRowValidity = (product, inventory) => {
        let invalid = false;

        if (!isProductRequiredFieldsFilled(product)) {
            invalid = 'FILL_REQUIRED_FIELDS';
        } else if (product.QTITINVENTRETI <= 0) {
            invalid = 'PRODUCT_QUANT_MUST_BE_GT_ZERO';
        }

        return existsProductInventory(product, inventory).then(exists => {
            if (exists) {
                invalid = 'ALREADY_EXISTS_PRODUCT_INVENTORY';
            }

            return !invalid ? Promise.resolve() : Promise.reject(invalid);
        });
    };

    /**
     * Check if product code and quantity are filled;
     * 
     * @param {Object} productRow Object containing product
     * data.
     * 
     * @returns {Boolean}
     */
    const isProductRequiredFieldsFilled = productRow => {
        return (productRow.CDPRODUTO !== undefined && productRow.CDPRODUTO !== null) &&
            (productRow.QTITINVENTRETI !== undefined && productRow.QTITINVENTRETI !== null);
    };

    /**
     * Check if current product code is already inserted
     * and related to current inventory.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise}
     */
    const existsProductInventory = (product, inventory) => {
        const productCode = product.CDPRODUTO;
        const unitCode = inventory.CDFILIAL;
        const inventoryCode = inventory.NRINVENTRETI;

        return util.getIDBVar('ProdutosInventario').then(productDataset => {
            if (productDataset && productDataset.length) {
                let exists = false;

                productDataset.forEach(prod => {

                    if (prod.CDFILIAL == unitCode &&
                        prod.CDPRODUTO == productCode &&
                        prod.NRINVENTRETI == inventoryCode) {

                        exists = true;
                    }

                });

                return exists;
            } else return false;
        });
    };

    /**
     * Insert new product register in 'ProdutosInventario'
     * dataset after make all validations.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise}
     */
    const insertNewProduct = (product, inventory) => {
        return setupProductRegisterToInsert(product, inventory)
            .then(() => {

                return util.insertIntoDataSource('ProdutosInventario', product).then(() => {
                    setNextInventoryProductCode(inventory.NRINVENTRETI);
                    return setInventoryIsEmptyObject(inventory, false);
                }).catch(errorCode => {
                    if (errorCode == 'IDB_KEY_NOT_EXISTS') {
                        return errorHandlerProductInsertion(product);
                    }
                });
            });
    };

    /**
     * Insert new product register in 'ProdutosInventario'
     * dataset after make all validations.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise} 
     */
    const setupProductRegisterToInsert = (product, inventory) => {
        const userData = getLocalVar('userData');

        return util.getIDBVar('nextInventoryProductCode').then(nextInventoryProductCode => {
            product.CDFILIAL = inventory.CDFILIAL;
            product.NRINVENTRETI = inventory.NRINVENTRETI;
            product.NRITINVENTRETI = nextInventoryProductCode[inventory.NRINVENTRETI];
            product.QTITINVENTRETI = product.QTITINVENTRETI;
            product.CDOPERINCLUSAO = userData.CDOPERADOR;
            product.NRORG = userData.NRORGTRAB;
            product.changed = false;
            product.__is_new = false;
            product.inserted = true;
        });
    };

    /**
     * Handle error after try to insert a product register.
     * Create an IDB register to store product list, then
     * try to insert it again.
     * 
     * @param {Object} productRow
     */
    const errorHandlerProductInsertion = (productRow) => {
        return util.setIDBVar('ProdutosInventario', [])
            .then(() => util.insertIntoDataSource('ProdutosInventario', productRow))
            .then(() => setInventoryIsEmptyObject(productRow, false));
    };

    /**
     * Remove inventory register and all products related 
     * with that.
     * 
     * @param {Object} itemToExclude Object containing
     * inventory data.
     * 
     * @returns {Promise}
     */
    this.removeInventoryRegister = itemToExclude => {
        util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmpty => {
            let keyToDelete = itemToExclude.NRINVENTRETI + '_' + itemToExclude.CDFILIAL;
            delete inventoryIsEmpty[keyToDelete];
            util.setIDBVar('inventoryIsEmpty', inventoryIsEmpty);
        });

        return removeAllInventoryProducts(itemToExclude).then(() => {
            return util.removeFromDataSource('Inventarios', inventory => {
                return inventory.CDFILIAL == itemToExclude.CDFILIAL &&
                    inventory.NRINVENTRETI == itemToExclude.NRINVENTRETI;
            });
        });
    };

    /**
     * Remove all products related with 
     * parameter's inventory.
     * 
     * @param {Object} inventory Object containing
     * inventory data.
     * 
     * @returns {Promise}
     */
    const removeAllInventoryProducts = inventory => {
        return util.removeFromDataSource('ProdutosInventario', product => {
            return product.CDFILIAL == inventory.CDFILIAL &&
                product.NRINVENTRETI == inventory.NRINVENTRETI;
        }).then(() => {
            setInventoryIsEmptyObject(inventory, true);
        });
    };

    /**
     * Check if inventory register has no product related
     * with it.
     * 
     * @param {Object} inventory Object containing
     * inventory data.
     * 
     * @returns {Promise}
     */
    this.checkIsInventoryEmpty = inventory => {
        return util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmptyObj => {
            let objKey = inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
            return inventoryIsEmptyObj[objKey] !== undefined &&
                inventoryIsEmptyObj[objKey];
        });
    };

    /**
     * Remove product register related with inventory register.
     * 
     * @param {Object} itemToExclude Object containing
     * inventory data.
     */
    this.removeInventoryProductRegister = productToExclude => {
        return util.removeFromDataSource('ProdutosInventario', product => {
            return product.CDFILIAL == productToExclude.CDFILIAL &&
                product.NRINVENTRETI == productToExclude.NRINVENTRETI &&
                product.CDPRODUTO == productToExclude.CDPRODUTO;
        }).then(newDataSourceLength => {
            if (newDataSourceLength === 0) {
                setInventoryIsEmptyObject(productToExclude, true);
            }

            let productList = templateManager.container.getWidget('inventoryProductList');
            productList.reload();
        });
    };

    /**
     * Remove user's selected inventorys.
     * 
     * @param {Array} inventoryArray
     * 
     * @returns {Promise}
     */
    this.removeSelectedInventorys = inventoryArray => {
        let mappedInventoryArray = inventoryArray.map(function(inventory) {
            return inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
        });

        return util.removeFromDataSource('Inventarios', inventory => {

            let inventoryKey = inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
            return mappedInventoryArray.includes(inventoryKey);

        }).then(util.removeFromDataSource('ProdutosInventario', product => {

            let productKey = product.NRINVENTRETI + '_' + product.CDFILIAL;
            return mappedInventoryArray.includes(productKey);

        })).then(util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmpty => {

            mappedInventoryArray.forEach(key => delete inventoryIsEmpty[key]);
            util.setIDBVar('inventoryIsEmpty', inventoryIsEmpty);

        }));
    };

    /**
     * Remove selected inventory's products registers.
     * 
     * @param {Array} productArray
     * 
     * @returns {Promise}
     */
    this.removeSelectedProducts = productArray => {
        let mappedProductArray = productArray.map(function(product) {
            return product.CDPRODUTO + '_' + product.NRINVENTRETI + '_' + product.CDFILIAL;
        });

        return util.removeFromDataSource('ProdutosInventario', product => {

            let productKey = product.CDPRODUTO + '_' + product.NRINVENTRETI + '_' + product.CDFILIAL;
            return mappedProductArray.includes(productKey);

        }).then(newDataSourceLength => {
            if (newDataSourceLength === 0) {
                setInventoryIsEmptyObject(productArray[0], true);
            }
        });
    };

    /**
     * Save edited inventory's product register. 
     * 
     * @param {Object} savedProduct Object containing
     * product data.
     * 
     * @returns {Promise}
     */
    this.saveEditedProduct = savedProduct => {
        return util.getIDBVar('ProdutosInventario').then(productDataset => {
            productDataset.forEach(product => {
                if (product.CDPRODUTO == savedProduct.CDPRODUTO &&
                    product.CDFILIAL == savedProduct.CDFILIAL &&
                    product.NRINVENTRETI == savedProduct.NRINVENTRETI) {

                    product.QTITINVENTRETI = savedProduct.QTITINVENTRETI;
                }
            });

            return util.setIDBVar('ProdutosInventario', productDataset);
        });
    };

    /**
     * Copy last exported inventory productset to the current
     * one.
     * 
     * @param {Object} currentInventory Current inventory register.
     * 
     * @returns {Promise}
     */
    this.copyLastExportedInventory = (currentInventory) => {
        return gestLastFilteredInventory().then(filteredInventoryProducts => {
            if (filteredInventoryProducts.length) {
                return removeAllInventoryProducts(currentInventory).then(() => {
                    return importProductsFromLastInventory(currentInventory, filteredInventoryProducts);
                });
            } else {
                return Promise.reject('HAS_NO_INVENTORY');
            }
        });
    };

    /**
     * Copy all products from last exported inventory to currentInventory.
     */
    const importProductsFromLastInventory = (currentInventory, filteredInventoryProducts) => {
        let insertPromises = [];

        filteredInventoryProducts.forEach(product => {
            product.QTITINVENTRETI = product.QTITINVENTRETI || 0;
            insertPromises.push(insertNewProduct(product, currentInventory));
        });

        return Promise.all(insertPromises);
    };

    /**
     * Get last inventory products filtered by unit
     * code.
     * 
     * @return {Promise}
     */
    const gestLastFilteredInventory = () => util.getIDBVar('UltimoInventario').then(lastInventory => {
        return lastInventory.filter(inventory => inventory.CDFILIAL == getLocalVar('userData').CDFILIAL);
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ManageInventoryService', ManageInventoryService);
});
