var PurchaseRequestService = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Get unit parameters inside 'Parametros'
     * dataSource according to unit code argument.
     * 
     * @param {String} unitCode Unit code
     * 
     * @return {Promise}
     */
    this.getUnitParameters = unitCode => SchoolMealsUtil.getIDBVar('Parametros').then(parameters => {
        return parameters.filter(param => param.CDFILIAL == unitCode);
    });

    /**
     * Set filter at product field in add new request
     * form.
     * 
     * @param {Object<Field>} productField
     * @param {Array} parameterArray
     */
    this.setAddProductFieldFilter = (productField, parameterArray) => {
        let dataSourceFilter = [];

        if (parameterArray.length && parameterArray[0].IDUTPRODPREF == 'N') {
            dataSourceFilter = [{
                name: 'TIPOPRODUTO',
                operator: '=',
                value: 'E'
            }];
        }
        dataSourceFilter.push({
            operator: '=',
            name: 'CDFILIAL',
            value: getLocalVar('userData').CDFILIAL
        });

        productField.dataSourceFilter = dataSourceFilter;
    };

    /**
     * Create new product purchase request with user's data
     * input if all data is valid to proceed.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    this.createNewProductPurchaseRequest = purchaseRequest => {
        return checkRequiredFieldInRequestForm(purchaseRequest)
            .then(requestDeliveryDateIsValid)
            .then(selectedQuantityIsValid)
            .then(alreadyExistsProductInSelectedDate)
            .then(createRequestRegisterIfNotExists)
            .then(insertProductPurchaseRequestRegister);
    };

    /**
     * Check if all required fields from purchase request
     * form are filled correctly.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const checkRequiredFieldInRequestForm = purchaseRequest => {
        if ((!purchaseRequest.DSJUSTOPER ||
                !purchaseRequest.CDPRODUTO ||
                (purchaseRequest.QTSOLIPROD === undefined || purchaseRequest.QTSOLIPROD === null) ||
                !purchaseRequest.DTPROGENTR) ||
            (purchaseRequest.PRODUTOEXTRA && !purchaseRequest.DSJUSTSOLEXT)) {

            return Promise.reject('FILL_REQUIRED_FIELDS');

        } else {
            return Promise.resolve(purchaseRequest);
        }
    };

    /**
     * Check if selected delivery date to purchase request 
     * is greater than today.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const requestDeliveryDateIsValid = purchaseRequest => {
        let selectedDate = moment(purchaseRequest.DTPROGENTR);
        let today = moment();

        if (selectedDate <= today) {
            return Promise.reject('SELECTED_DATE_HAVE_BE_GREATER_THAN_TODAY');
        } else {
            return Promise.resolve(purchaseRequest);
        }
    };

    /**
     * Check if selected quantity is greater than 0.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const selectedQuantityIsValid = purchaseRequest => {
        if (purchaseRequest.QTSOLIPROD > 0) {
            return Promise.resolve(purchaseRequest);
        } else {
            return Promise.reject('SELECTED_QUANTITY_HAVE_BE_GREATER_THAN_0');
        }
    };

    /**
     * Check if already exists product related with selected
     * date.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const alreadyExistsProductInSelectedDate = purchaseRequest => {
        let productCode = purchaseRequest.CDPRODUTO;
        let deliveryDate = purchaseRequest.DTPROGENTR;
        let unitCode = getLocalVar('userData').CDFILIAL;

        return SchoolMealsUtil.getIDBVar('ProdutosSolicitacao').then(products => {
            let exists = products.filter(product =>
                product.CDPRODUTO == productCode &&
                product.DTPROGENTR == deliveryDate &&
                product.CDFILSOLI == unitCode).length;

            if (exists) {
                return Promise.reject('ALREADY_EXISTS_PRODUCT_FOR_DELIVERY_DATE');
            } else {
                return Promise.resolve(purchaseRequest);
            }
        });
    };

    /**
     * Create new a purchase register into "NovaSolicitacao"
     * datasource if it not exists for current unit.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const createRequestRegisterIfNotExists = purchaseRequest => {
        return SchoolMealsUtil.getIDBVar('NovaSolicitacao').then(requestList => {
            if (!existsRequestRegisterForCurrentUnit(requestList)) {
                let requestRegister = buildNewRequestRegister();
                return SchoolMealsUtil.insertIntoDataSource('NovaSolicitacao', requestRegister).then(() => purchaseRequest);
            } else {
                return Promise.resolve(purchaseRequest);
            }
        });
    };

    /**
     * Check if already an existent purchase register inserted
     * for current unit.
     * 
     * @param {Array} requestList 
     * 
     * @return {Boolean}
     */
    const existsRequestRegisterForCurrentUnit = requestList => {
        return requestList.find(request => {
            return request.CDFILLANCA === getLocalVar('userData').CDFILLANCA &&
                request.CDFILSOLI === getLocalVar('userData').CDFILIAL;
        }) !== undefined;
    };

    /**
     * Build new purchase request register according to current
     * unit parameters to insert into 'NovaSolicitacao' dataSource.
     * 
     * @returns {Object}
     */
    const buildNewRequestRegister = () => {
        return {
            CDFILLANCA: getLocalVar('userData').CDFILLANCA,
            CDFILSOLI: getLocalVar('userData').CDFILIAL,
            NRSOLICMP: null,
            DTSOLCMP: moment().format('DD/MM/YYYY'),
            IDTIPLAN: 'M',
            IDSOLEXTRA: "N",
            CDFILCENT: getLocalVar('userData').CDFILCENTCMP,
            CDOPERLANC: getLocalVar('userData').CDOPERADOR,
            DTHRLANCSOL: moment().format('YYYY-MM-DD HH:mm:ss'),
            QTITEMSOLIC: null,
            NRORG: getLocalVar('userData').NRORG,
            inserted: true
        };
    };

    /**
     * Insert a product purchase request register in correct
     * related request register.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const insertProductPurchaseRequestRegister = purchaseRequest => {
        let productRequestObject = buildProductRequestRegisterToInsert(purchaseRequest);
        return SchoolMealsUtil.insertIntoDataSource('ProdutosSolicitacao', productRequestObject).then(() => {
            return createLog_MOVOPSOL(productRequestObject, 'I', 0, productRequestObject.QTSOLIPROD);
        });
    };

    /**
     * Insert a register into MOVOPSOL dataSource to store
     * operation log.
     * 
     * @param {Object} productRequestObject 
     * @param {String} operationCode
     * @param {String} previousQuantity
     * @param {String} afterQuantity
     * 
     * @returns {Promise}
     */
    const createLog_MOVOPSOL = (productRequestObject, operationCode, previousQuantity, afterQuantity) => {
        return SchoolMealsUtil.insertIntoDataSource('Movopsol', {
            CDFILSOLMOV: productRequestObject.CDFILSOLI,
            CDFILLANMOV: productRequestObject.CDFILLAN,
            NRSOLIMOV: productRequestObject.NRSOLICMP,
            CDPRODUTO: productRequestObject.CDPRODUTO,
            DTPROGENTR: productRequestObject.DTPROGENTR,
            DTOPERSOL: moment().format("YYYY-MM-DD HH:mm:ss"),
            CDOPERADOR: getLocalVar('userData').CDOPERADOR,
            DSJUSTOPER: productRequestObject.DSJUSTOPER || null,
            DSJUSTSOLEXT: productRequestObject.DSJUSTSOLEXT || null,
            QTANTOPER: previousQuantity,
            QTDEPOPER: afterQuantity,
            NRORG: productRequestObject.NRORG,
            IDOPERAMOVS: operationCode,
            IDITMSOLEXTRA: productRequestObject.IDITMSOLEXTRA,
            inserted: true
        });
    };

    /**
     * Build an object containing a new product purchase register
     * data to insert into 'ProdutosSolicitacao' dataSource.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Object}
     */
    const buildProductRequestRegisterToInsert = purchaseRequest => {
        return {
            CDFILLAN: getLocalVar('userData').CDFILLANCA,
            CDFILSOLI: getLocalVar('userData').CDFILIAL,
            CDPRODUTO: purchaseRequest.CDPRODUTO,
            DSJUSTOPER: purchaseRequest.DSJUSTOPER,
            DTPROGENTR: purchaseRequest.DTPROGENTR,
            DTUTILIZA: purchaseRequest.DTPROGENTR,
            NMPRODUTO: purchaseRequest.NMPRODUTO,
            NRORG: getLocalVar('userData').NRORG,
            CDFILLANCA: getLocalVar('userData').CDFILLANCA,
            NRSOLICMP: null,
            IDITMSOLEXTRA: purchaseRequest.PRODUTOEXTRA ? 'S' : 'N',
            DSJUSTSOLEXT: purchaseRequest.DSJUSTSOLEXT || null,
            QTAJUSTADA: null,
            QTSOLIPROD: purchaseRequest.QTSOLIPROD,
            SGUNIDADE: purchaseRequest.SGUNIDADE,
            IDSOLIAPROV: 'N',
            CDFILIOPELOG: null,
            inserted: true
        };
    };

    /**
     * Remove swiped product register if is valid to.
     * If is not, return a message code related
     * what's wrong with operation.
     */
    this.removeValidProductPurchaseRequest = itemToExclude => {
        return getUnitParametersByPurchasingCentral()
            .then(parameters => isValidProductToExclude(parameters, itemToExclude))
            .then(isValid => {
                if (isValid) {
                    SchoolMealsUtil.insertIntoDataSource('ProdutosExcluidos', itemToExclude);
                    createLog_MOVOPSOL(itemToExclude, 'E', itemToExclude.QTSOLIPROD || itemToExclude.QTAJUSTADA, 0);
                    return SchoolMealsUtil.removeFromDataSource('ProdutosSolicitacao', product => {
                        return product.CDPRODUTO == itemToExclude.CDPRODUTO &&
                            product.DTPROGENTR == itemToExclude.DTPROGENTR &&
                            product.CDFILSOLI == getLocalVar('userData').CDFILIAL;
                    });
                } else {
                    return Promise.reject();
                }
            });
    };

    /** 
     * Get unit parameters inside 'Parametros'
     * dataSource according to unit code and central
     * code. 
     * 
     * @returns {Promise}
     */
    const getUnitParametersByPurchasingCentral = () => {
        let unitCode = getLocalVar('userData').CDFILIAL;
        let centralCode = getLocalVar('userData').CDFILCENTCMP;

        return this.getUnitParameters(unitCode).then(parameters => {
            return parameters.find(parameter => parameter.CENTRAL == centralCode);
        });
    };

    /**
     * Check if parameter product is valid to exclude according
     * to unit and purchasing central parameters.
     * 
     * If delivery date is expired, so product just can be edited
     * or removed if it does request approvation and central 
     * parameters allow to edit approved products.
     * 
     * If date is not expired already, so product can be edited/removed
     * if it doest not request to be approved.
     * 
     * @param {Object} parameters Filtered unit and purchasing
     * central parameters object.
     * @param {Object} itemToExclude Object containing data of product
     * to exclude.
     */
    const isValidProductToExclude = (parameters, itemToExclude) => {
        let isValid = itemToExclude.IDSOLIAPROV != 'S';
        let productDeliveryDate = moment(itemToExclude.DTPROGENTR);
        let today = moment();

        if (!isValid) {
            return Promise.reject("PRODUCT_CANNOT_BE_REMOVED_REQUEST_APPROVATION");
        } else if (productDeliveryDate <= today || (itemToExclude.IDSOLIAPROV == 'S' && parameters.ALTERAR_PRODUTO_APROVADO == 'N')) {
            return Promise.reject("PRODUCT_CANNOT_BE_REMOVED_EXPIRED_DELIVERY");
        } else {
            return Promise.resolve(parameters, itemToExclude);
        }

    };

    /**
     * Check if parameter product is valid to edit according
     * to unit and purchasing central parameters.
     * 
     * If delivery date is expired, so product just can be edited
     * or removed if it does request approvation and central 
     * parameters allow to edit approved products.
     * 
     * If date is not expired already, so product can be edited/removed
     * if request is already created.
     * 
     * @param {Object} itemToEdit Object containing data of product
     * to Edit.
     * 
     * @returns {Promise}
     */
    this.isValidProductToEdit = (itemToEdit) => {
        let productDeliveryDate = moment(itemToEdit.DTPROGENTR);
        let today = moment();

        return getUnitParametersByPurchasingCentral().then(parameters => {
            return productDeliveryDate > today &&
                !(itemToEdit.IDSOLIAPROV == 'S' && parameters.ALTERAR_PRODUTO_APROVADO == 'N') &&
                (itemToEdit.NRSOLICMP == null);
        });
    };

    /**
     * Remove selected products in purchase request list.
     * 
     * @param {Array} selectedRegisters Checked products in
     * product purchase request list.
     * 
     * @returns {Promise}
     */
    this.removeSelectedProductsIfAreValid = selectedRegisters => {
        return getUnitParametersByPurchasingCentral()
            .then(parameters => validateSelectedProductsToExclude(parameters, selectedRegisters))
            .then(isValid => {
                if (isValid) {
                    insertSelectedProductsIntoRemovedDataSet(selectedRegisters);

                    selectedRegisters.forEach(purchaseRequest => {
                        createLog_MOVOPSOL(purchaseRequest, 'E', purchaseRequest.QTSOLIPROD || purchaseRequest.QTAJUSTADA, 0);
                    });

                    let mappedRegisters = selectedRegisters.map(purchaseRequest => {
                        return purchaseRequest.CDFILSOLI + '_' + purchaseRequest.CDPRODUTO + '_' + purchaseRequest.DTPROGENTR;
                    });
                    return SchoolMealsUtil.removeFromDataSource('ProdutosSolicitacao', product => {
                        let objKey = product.CDFILSOLI + '_' + product.CDPRODUTO + '_' + product.DTPROGENTR;
                        return mappedRegisters.includes(objKey);
                    });

                } else {
                    return Promise.reject();
                }
            });
    };

    /**
     * Insert selected products to remove into
     * 'ProdutosExcluidos' dataSource
     * 
     * @param {*} selectedProducts 
     */
    const insertSelectedProductsIntoRemovedDataSet = selectedProducts => {
        SchoolMealsUtil.getIDBVar('ProdutosExcluidos').then(productDataSource => {
            productDataSource = productDataSource.concat(selectedProducts);
            SchoolMealsUtil.setIDBVar('ProdutosExcluidos', productDataSource);
        });
    };

    /**
     * Check if all selected products are valid to exclude
     * according to individual product verification to remove.
     * 
     * @param {Object} parameters Filtered unit and purchasing
     * central parameters object.
     * @param {Array} selectedRegisters Checked products in
     * product purchase request list.
     * 
     * @returns {Promise}
     */
    const validateSelectedProductsToExclude = (parameters, selectedRegisters) => {
        let validationPromiseArray = [];

        selectedRegisters.forEach(product => {
            validationPromiseArray.push(isValidProductToExclude(parameters, product));
        });

        return Promise.all(validationPromiseArray).then(isValidArray => !isValidArray.includes(false));
    };

    /**
     * Save edited purchase request data.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchase request data
     */
    this.saveEditedProductPurchaseRequest = purchaseRequest => {
        let parameters = {};
        return getUnitParametersByPurchasingCentral()
            .then(unitParam => {
                parameters = unitParam;
                checkRequiredFieldInRequestForm(purchaseRequest);
            })
            .then(() => adjustedQuantityIsValid(purchaseRequest, parameters))
            .then(saveProductChangeOnDataSource)
            .then(() => createLog_MOVOPSOL(purchaseRequest, 'A', purchaseRequest.QTSOLIPROD, purchaseRequest.QTAJUSTADA));
    };

    /**
     * Check if adjusted quantity is greater than 0 and if
     * is not greater then previous quantity.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * @param {Object} unitParam Object containing
     * unit parameters.
     * 
     * @returns {Promise}
     */
    const adjustedQuantityIsValid = (purchaseRequest, unitParam) => {
        if (purchaseRequest.QTAJUSTADA <= 0) {
            return Promise.reject('ADJUSTED_QUANTITY_HAVE_BE_GREATER_THAN_0');
        } else if (purchaseRequest.QTAJUSTADA > purchaseRequest.QTSOLIPROD &&
            unitParam.IDBLOQALTSOLAVR == 'S' && !!purchaseRequest.NRSOLICMP) {
            return Promise.reject('ADJUSTED_QUANTITY_HAVE_BE_GREATER_THAN_PREVIOUS_QUANT');
        } else {
            return Promise.resolve(purchaseRequest, unitParam);
        }
    };

    /**
     * Set changes on product to indexed db register.
     * 
     * @param {Object} purchaseRequest 
     * @param {Object} unitParam 
     */
    const saveProductChangeOnDataSource = (purchaseRequest, unitParam) => {
        if (purchaseRequest.IDSOLIAPROV == 'S' && unitParam.DESAPROVAR_PRODUTO_ALTERADO == 'S') {
            purchaseRequest.IDSOLIAPROV = 'N';
        }
        purchaseRequest.NMSOLIAPROV = purchaseRequest.IDSOLIAPROV == 'S' ? 'Sim' : 'Não';

        return SchoolMealsUtil.getIDBVar('ProdutosSolicitacao').then(productList => {
            productList.forEach(product => {
                if (product.CDPRODUTO == purchaseRequest.CDPRODUTO && product.DTPROGENTR == purchaseRequest.DTPROGENTR &&
                    product.CDFILSOLI == purchaseRequest.CDFILSOLI) {

                    Object.assign(product, purchaseRequest, {
                        edited: true
                    });
                }
            });

            return SchoolMealsUtil.setIDBVar('ProdutosSolicitacao', productList);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('PurchaseRequestService', PurchaseRequestService);
});
