var StaffManagementService = function(restEngine, SchoolMealsUtil) {

    /**
     * Check if logged unit is a meal center.
     * 
     * @returns {Boolean}
     */
    this.isLoggedUnitMealsCenter = () => {
        let userData = getLocalVar('userData');
        return userData.CDFILIAL == userData.CDFILCENTPME;
    };

    /**
     * Check if logged unit has a meal center.
     * 
     * @returns {Boolean}
     */
    this.loggedUnitHasMealsCenter = () => {
        let userData = getLocalVar('userData');
        return !!userData.CDFILCENTPME;
    };

    /**
     * Check if has staff registers in selected
     * import date.
     * 
     * @returns {Promise}
     */
    this.checkIfHasStaffInPeriod = () => {
        return SchoolMealsUtil.getIDBVar('TiposCardapioEfetivos').then(staff => {
            if (staff && staff.length) {
                return Promise.resolve(staff);
            } else {
                return Promise.reject('CANNOT_MANAGE_STAFF_HAS_NO_STAFF_IN_PERIOD');
            }
        });
    };

    /**
     * Check if has staff registers in logged unit.
     * 
     * @returns {Promise}
     */
    this.checkIfHasStaffInUnit = () => {
        let unitCode = getLocalVar('userData').CDFILIAL;

        return SchoolMealsUtil.getIDBVar('TiposCardapioEfetivos').then(staff => {
            staff = staff.filter(reg => reg.CDFILIAL == unitCode);

            if (staff && staff.length) {
                return Promise.resolve(staff);
            } else {
                return Promise.reject('CANNOT_MANAGE_STAFF_HAS_NO_STAFF_IN_UNIT');
            }
        });
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service name using 
     * alphabetical order.
     * 
     * @param {DataSource} dataSource Widget's dataSource
     */
    this.sortServiceListByServiceName = dataSource => {
        dataSource.orderingAscAlpha = !dataSource.orderingAscAlpha;

        dataSource.data.sort((a, b) => {
            let nameA = a.NMSERVICO.toLowerCase();
            let nameB = b.NMSERVICO.toLowerCase();
            let order = 0;

            order = (nameA < nameB) ? -1 : 1;

            if (!dataSource.orderingAscAlpha) {
                order *= -1;
            }

            return order;
        });
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service name using 
     * alphabetical order.
     * 
     * @param {DataSource} dataSource Widget's dataSource
     */
    this.sortServiceListByDate = dataSource => {
        dataSource.orderingAscDate = !dataSource.orderingAscDate;

        dataSource.data.sort((a, b) => {
            let dateA = moment(a.DTCARDIAPME, 'DD/MM/YYYY');
            let dateB = moment(b.DTCARDIAPME, 'DD/MM/YYYY');
            let order = 0;

            order = (dateA < dateB) ? -1 : 1;

            if (!dataSource.orderingAscDate) {
                order *= -1;
            }

            return order;
        });
    };

    /**
     * Save staff edited register.
     * 
     * @param {Object} staffData Staff edited data.
     * 
     * @returns {Promise}
     */
    this.saveStaffEdit = staffData => {
        return SchoolMealsUtil.getIDBVar('ServicosCardapioEfetivos')
            .then(serviceList => {
                serviceList.forEach(service => {
                    if (service.CDSERVFILI == staffData.CDSERVFILI &&
                        service.CDFILIAL == staffData.CDFILIAL &&
                        service.DTCARDIAPME == staffData.DTCARDIAPME) {

                        service.QTCLIEFECPME = staffData.QTCLIEFECPME;
                        service.QTCOMEFECPME = staffData.QTCOMEFECPME;
                        service.QTDOAEFECPME = staffData.QTDOAEFECPME;
                        service.QTFUNEFECPME = staffData.QTFUNEFECPME;

                        service.edited = true;
                    }
                });

                saveStaffQuantityInMainClient(staffData);
                return SchoolMealsUtil.setIDBVar('ServicosCardapioEfetivos', serviceList);
            });
    };

    /**
     * Save service staff quantities in main client register.
     * 
     * @param {Object} staffData Staff edited data.
     * 
     * @returns {Promise}
     */
    const saveStaffQuantityInMainClient = staffData => {
        return SchoolMealsUtil.getIDBVar('ClientesDoServicoEfetivos')
            .then(clientList => {
                let staffClientList = clientList.filter(client => {
                    return client.CDFILIAL == staffData.CDFILIAL &&
                        client.DTEFETCLPME == staffData.DTCARDIAPME &&
                        client.CDSERVFILI == staffData.CDSERVFILI &&
                        client.CDTIPOCARD == staffData.CDTIPOCARD;
                });

                if (staffClientList.length == 1) {
                    staffClientList[0].QTDOAEFCLPME = staffData.QTDOAEFECPME;
                    staffClientList[0].QTFUNEFCLPME = staffData.QTFUNEFECPME;
                    staffClientList[0].QTCOMEFCLPME = staffData.QTCLIEFECPME;
                    staffClientList[0].TOTAL = staffData.QTCOMEFECPME;
                    staffClientList[0].edited = true;
                    return SchoolMealsUtil.setIDBVar('ClientesDoServicoEfetivos', clientList);
                }

            });
    };

    /**
     * Save all staff client quantities on 
     * 'ClientesDoServicoEfetivos' dataSource if
     * is valid.
     * Client data is valid if sum of all client's
     * quantities is equal to service total staff
     * quantity.
     * 
     * @param {Array} staffClientListData
     * @param {Object} staffRegister
     * 
     * @returns {Promise}
     */
    this.saveStaffClientQuantities = (staffClientListData, staffRegister) => {
        return isValidClientStaffQuantities(staffClientListData, staffRegister)
            .then(() => saveStaffClientRegisters(staffClientListData));
    };

    /**
     * Client data is valid if sum of all client's
     * quantities is equal to service total staff
     * quantity.
     * 
     * @param {Array} staffClientListData
     * @param {Object} staffRegister
     * 
     * @returns {Promise}
     */
    const isValidClientStaffQuantities = (staffClientListData, staffRegister) => {
        let sum = 0;

        if (staffClientListData.length > 1) {
            sum = staffClientListData.reduce((a, b) => a.TOTAL + b.TOTAL);
        } else if (staffClientListData.length) {
            sum = staffClientListData[0].TOTAL;
        }

        if (sum == staffRegister.QTCOMEFECPME) {
            return Promise.resolve();
        } else {
            return Promise.reject('STAFF_CLIENT_SUM_NOT_EQUAL');
        }
    };

    /**
     * Save staff client registers on 'ClientesDoServicoEfetivos'
     * dataSource.
     * 
     * @param {Array} staffClientListData 
     * 
     * @returns {Promise}
     */
    const saveStaffClientRegisters = staffClientListData => {
        return SchoolMealsUtil.getIDBVar('ClientesDoServicoEfetivos').then(staffDataSet => {
            staffClientListData.forEach(client => {
                let clientReg = staffDataSet.find(staffClient => {
                    return client.CDFILIAL == staffClient.CDFILIAL &&
                        client.DTEFETCLPME == staffClient.DTEFETCLPME &&
                        client.CDSERVFILI == staffClient.CDSERVFILI &&
                        client.CDTIPOCARD == staffClient.CDTIPOCARD &&
                        client.CDCLIENTE == staffClient.CDCLIENTE;
                });

                Object.assign(clientReg, client, {
                    edited: true
                });
            });

            return SchoolMealsUtil.setIDBVar('ClientesDoServicoEfetivos', staffDataSet);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('StaffManagementService', StaffManagementService);
});
