let UIManager = function(eventAggregator, templateManager, ScreenService) {

    window._UIM = this;
    /**
     * @type {Array<String|Function>}
     */
    let stateStack = [];

    Object.defineProperty(window, '_stack', {
        get: () => stateStack
    });

    let isListenerAttached = false;
    let activeWidget;
    let sf = this;
    /**
     * Called every time a container is open
     */
    this.init = () => {
        // stateStack = [];
        activeWidget = this.activeWidget = templateManager.container.getActiveWidget();
    };

    if (!isListenerAttached) {
        eventAggregator.subscribe('onContainerOpen', this.init);
        isListenerAttached = true;
    }

    /**
     * Displays the given widget
     * @param {ZhWidgetOjbect} widgetObject
     * @param {boolean} [shouldReload] 
     * @param {function} [callbackFunction] 
     */
    this.goToWidget = (widgetObject, shouldReload = false, callbackFunction = null) => {
        if (widgetObject && widgetObject.name !== activeWidget.name) {
            const previousActiveWidget = activeWidget.name;
            widgetObject.activate();
            activeWidget.isVisible = false;
            widgetObject.isVisible = true;
            activeWidget = widgetObject;
            stateStack.push({
                type: "widget",
                name: activeWidget.name,
                parentWidget: previousActiveWidget,
                parentScrollInfo: _getScrollPosition(),
                callbackFunction: callbackFunction
            });
            if (shouldReload) {
                return widgetObject.reload();
            } else {
                return Promise.resolve();
            }
        }
    };

    /**
     * Tryes to find a widget by it's name and display it
     * @param {string} widgetName 
     * @param {boolean} shouldReload 
     */
    this.goToWidgetByName = (widgetName, shouldReload = false, shouldCarryCurRow = false) => {
        let widgetToOpen = templateManager.container.getWidget(widgetName);
        if (widgetToOpen && widgetToOpen.name !== activeWidget.name) {
            widgetToOpen.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetToOpen.isVisible = true;
            if (shouldCarryCurRow) {
                widgetToOpen.currentRow = activeWidget.currentRow;
            }
            activeWidget = widgetToOpen;

            if (shouldReload) {
                return widgetToOpen.reload();
            }
        } else {
            console.log('%can attempt to open the widget: ' + widgetName + " was made. But this widget wasn't found", "font-size: 15px");
        }
    };

    /**
     * Displays a widget that is already in the stack 
     * and removes all others in the path
     * 
     * homeScreen -> list -> listItem -> listItemDetails
     * 
     * goBackToStackedWidget('list')
     * 
     * homeScreen -> list
     *  
     * @param {string} widgetName 
     */
    this.goBackToStackedWidget = widgetName => {
        let widgetToGoBack = templateManager.container.getWidget(widgetName);
        let widgetNameIndex = stateStack.findIndex(widget => widget.name === widgetName);
        if (widgetToGoBack && widgetNameIndex !== -1 && stateStack.length > 0) {
            stateStack = stateStack.slice(0, widgetNameIndex + 1);
            widgetToGoBack.activate();
            activeWidget.isVisible = false;
            widgetToGoBack.isVisible = true;
            activeWidget = widgetToGoBack;
        }
    };

    /**
     * Displays the last widget in the stack
     * or triggers the last callback state in the stack 
     */
    this.goBack = () => {
        if (stateStack.length > 0) {
            if (typeof stateStack[stateStack.length - 1] === 'string') {
                let widgetToGoBackName = stateStack.pop();
                let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackName);
                widgetToGoBack.activate();
                widgetToGoBack.isVisible = true;
                activeWidget.isVisible = false;

                activeWidget = widgetToGoBack;
            } else if (typeof stateStack[stateStack.length - 1] === 'function') {
                (stateStack.pop())();
            } else if (typeof stateStack[stateStack.length - 1] === 'object') {
                let currentState = stateStack.pop();
                if (currentState.type === 'container') {
                    let parentState = stateStack[stateStack.length - 1];
                    if (!parentState || (parentState.type === 'container' && !parentState.skipContainer)) {
                        let parentContainerArguments = ((parentState && parentState.args) || {});
                        ScreenService.openWindow(currentState.parentContainer, parentContainerArguments);
                    } else {
                        this.goBack();
                    }
                    _scrollIntoPosition(currentState && currentState.parentScrollInfo, 100);
                } else if (currentState.type === 'widget') {
                    let widgetToGoBackData = currentState;
                    let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackData.parentWidget);
                    widgetToGoBack.activate();
                    widgetToGoBack.isVisible = true;
                    activeWidget.isVisible = false;
                    activeWidget = widgetToGoBack;
                    _scrollIntoPosition(currentState && currentState.parentScrollInfo, 100);
                } else if (currentState.type === 'popUp') {
                    let widgetToGoBackData = currentState;
                    let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackData.parentWidget);
                    widgetToGoBack.activate();
                    widgetToGoBack.isVisible = true;
                    // activeWidget.isVisible = false;

                    activeWidget = widgetToGoBack;
                    ScreenService.closePopup();
                    _scrollIntoPosition(currentState && currentState.parentScrollInfo, 100);
                }
            }
        }
    };

    /**
     * hides the editor of the active widget, (if this widget is editable)
     */
    this.hideEdit = () => {
        if (activeWidget.hasOwnProperty('isEditing')) {
            this.allowScroll();
            activeWidget.isEditing = false;
            // activeWidget.restoreHeaderActions();
        }
    };

    this.preventScroll = () => document.body.classList.add('no-scroll');

    this.allowScroll = () => document.body.classList.remove('no-scroll');

    /**
     * @param {Function} callback
     */
    this.pushState = (callback) => stateStack.push(callback);

    this.openContainer = (containerName, args, keepInHistory = true) => {
        ScreenService.openWindow(containerName, args);
        // if (keepInHistory) {
        let previouseContainer = templateManager.container.name;
        // stateStack.push(() => {
        //     ScreenService.openWindow(previouseContainer);
        // });
        stateStack.push({
            type: "container",
            name: containerName,
            parentContainer: previouseContainer,
            skipContainer: !keepInHistory,
            args: args || {},
            parentScrollInfo: _getScrollPosition()
        });
        // }
    };

    this.clearStack = () => stateStack = [];

    this.popState = () => stateStack.pop();

    this.openPopUp = (widgetToOpen) => {
        const previousActiveWidget = templateManager.container.getActiveWidget().name;
        ScreenService.openPopup(widgetToOpen);
        stateStack.push({
            type: "popUp",
            name: widgetToOpen.name,
            parentWidget: previousActiveWidget,
            parentScrollInfo: _getScrollPosition()
        });
        // widgetToOpen.isVisible = true;
    };

    let _getScrollPosition = () => {
        let documentScroll = document.documentElement.scrollTop;
        let scrollInfo = {
            elementSelector: null,
            scrollValue: null
        };
        if (documentScroll !== 0) {
            scrollInfo.scrollValue = documentScroll;
            scrollInfo.elementSelector = 'html';
        } else {
            let scrollable = document.querySelector('.scrollable');
            if (scrollable && scrollable.scrollTop !== 0) {
                scrollInfo.scrollValue = scrollTop;
                scrollInfo.elementSelector = '.scrollable';
            }
        }
        return scrollInfo;
    };

    let _scrollIntoPosition = (scrollInfo, delay = 50) => {
        if (scrollInfo && scrollInfo.elementSelector && scrollInfo.scrollValue) {
            let element = document.querySelector(scrollInfo.elementSelector);
            setTimeout(() => element.scrollTop = scrollInfo.scrollValue, delay);
        }
    };

    this.openHeaderMenu = () => templateManager.container.openHeaderMenu();

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UIManager', UIManager);
});
